@@ -36,7 +36,7 @@
  *									   *
  ****************************************************************************/
 
-#include "myodbc3.h"
+#include "driver.h"
 #include <locale.h>
 
 /*
@@ -100,18 +100,21 @@ static const char *find_used_table(STMT *stmt)
   if we found the start of the string, return it
 */
 
-static const char *mystr_get_prev_token( const char **query, const char *start )
+static const char *mystr_get_prev_token(CHARSET_INFO *charset,
+                                        const char **query, const char *start)
 {
-    const char *pos = *query;
+  const char *pos= *query;
 
-    do
-    {
-        if ( pos == start )
-            return ( *query = start );     /* Return start of string */
-    } while ( !isspace( *(--pos) ) ) ;
-    *query = pos;      /* Remember pos to space */
+  do
+  {
+    if (pos == start)
+      return (*query = start);     /* Return start of string */
+    pos--;
+  } while (*pos < 0 || !my_isspace(charset, *pos)) ;
 
-    return ( pos + 1 );   /* Return found token */
+  *query= pos;      /* Remember pos to space */
+
+  return pos + 1;   /* Return found token */
 }
 
 
@@ -129,14 +132,18 @@ char *check_if_positioned_cursor_exists(STMT *pStmt, STMT **pStmtCursor)
   if (pStmt->query && pStmt->query_end)
   {
     const char *pszQueryTokenPos= pStmt->query_end;
-    const char *pszCursor= mystr_get_prev_token((const char**)&pszQueryTokenPos,
+    const char *pszCursor= mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
+                                                (const char**)&pszQueryTokenPos,
                                                 pStmt->query);
 
-    if (!myodbc_casecmp(mystr_get_prev_token(&pszQueryTokenPos,
+    if (!myodbc_casecmp(mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
+                                             &pszQueryTokenPos,
                                              pStmt->query),"OF",2) &&
-        !myodbc_casecmp(mystr_get_prev_token(&pszQueryTokenPos,
+        !myodbc_casecmp(mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
+                                             &pszQueryTokenPos,
                                              pStmt->query),"CURRENT",7) &&
-        !myodbc_casecmp(mystr_get_prev_token(&pszQueryTokenPos,
+        !myodbc_casecmp(mystr_get_prev_token(pStmt->dbc->ansi_charset_info,
+                                             &pszQueryTokenPos,
                                              pStmt->query),"WHERE",5) )
     {
       LIST *list_element;
@@ -352,9 +359,9 @@ static SQLRETURN update_status(STMT FAR *stmt, SQLUSMALLINT status)
       SQLBulkOperations(), so we don't have to worry about the row status
       set by SQLExtendedFetch().
     */
-    else if ( stmt->stmt_options.rowStatusPtr )
+    else if ( stmt->ird->array_status_ptr )
     {
-        SQLUSMALLINT *ptr= stmt->stmt_options.rowStatusPtr+stmt->current_row;
+        SQLUSMALLINT *ptr= stmt->ird->array_status_ptr+stmt->current_row;
         SQLUSMALLINT *end= ptr+stmt->affected_rows;
 
         for ( ; ptr != end ; ptr++ )
@@ -381,9 +388,9 @@ static SQLRETURN update_setpos_status(STMT FAR *stmt, SQLINTEGER irow,
     If all rows successful, then only update status..else
     don't update...just for the sake of performance..
   */
-  if (stmt->stmt_options.rowStatusPtr)
+  if (stmt->ird->array_status_ptr)
   {
-    SQLUSMALLINT *ptr= stmt->stmt_options.rowStatusPtr;
+    SQLUSMALLINT *ptr= stmt->ird->array_status_ptr;
     SQLUSMALLINT *end= ptr+rows;
 
     for ( ; ptr != end; ptr++)
@@ -408,22 +415,26 @@ static SQLRETURN update_setpos_status(STMT FAR *stmt, SQLINTEGER irow,
   @purpose : copy row buffers to statement
 */
 
-static SQLRETURN copy_rowdata(STMT FAR *stmt, PARAM_BIND  param,
-                              NET **net, SQLCHAR **to)
+static SQLRETURN copy_rowdata(STMT FAR *stmt, DESCREC *aprec,
+                              DESCREC *iprec, NET **net, SQLCHAR **to)
 {
+    SQLRETURN rc;
     SQLCHAR *orig_to= *to;
     /* Negative length means either NULL or DEFAULT, so we need 7 chars. */
-    SQLUINTEGER length= (*param.actual_len > 0 ? *param.actual_len + 1 : 7);
+    SQLUINTEGER length= (*aprec->octet_length_ptr > 0 ?
+                         *aprec->octet_length_ptr + 1 : 7);
 
     if ( !(*to= (SQLCHAR *) extend_buffer(*net,(char*) *to,length)) )
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
-    if ( !(*to= (SQLCHAR*) insert_param(stmt->dbc, (char*) *to, &param)) )
-        return set_error(stmt,MYERR_S1001,NULL,4001);
+    rc= insert_param(stmt, (char**) to, stmt->apd, aprec, iprec, 0);
+    if (!SQL_SUCCEEDED(rc))
+        return rc;
 
     /* We have to remove zero bytes or we have problems! */
     while ( (*to > orig_to) && (*((*to) - 1) == (SQLCHAR) 0) ) (*to)--;
 
+    /* insert "," */
     if (!(*to= (SQLCHAR *)add_to_buffer(*net, (char *)*to, ",", 1)))
         return set_error(stmt,MYERR_S1001,NULL,4001);
 
@@ -455,30 +466,6 @@ static SQLRETURN exec_stmt_query(STMT FAR *stmt,char *query,
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : copy row buffers to statement
-*/
-
-static SQLRETURN copy_field_data(STMT FAR *stmt, PARAM_BIND *param,
-                                 NET **net, SQLCHAR **to)
-{
-    SQLUINTEGER length= *(param->actual_len)+5;
-
-    if ( !(*to= (SQLCHAR*) extend_buffer(*net, (char*) *to,length)) )
-        return set_error(stmt,MYERR_S1001,NULL,4001);
-
-    if ( !(*to= (SQLCHAR*) insert_param(stmt->dbc, (char*) *to, param)) )
-        return set_error(stmt,MYERR_S1001,NULL,4001);
-
-    /* Insert " AND ", where clause with multiple search */
-    if (!(*to= (SQLCHAR *)add_to_buffer(*net, (char *)*to, " AND ", 5)))
-        return set_error(stmt,MYERR_S1001,NULL,4001);
-
-    return SQL_SUCCESS;
-}
-
-
 /*
   @type    : myodbc3 internal
   @purpose : copies field data to statement
@@ -488,27 +475,34 @@ static my_bool insert_field(STMT FAR *stmt, MYSQL_RES *result,
                             DYNAMIC_STRING *dynQuery,
                             SQLUSMALLINT nSrcCol)
 {
-    PARAM_BIND  param;
+    DESCREC aprec_, iprec_;
+    DESCREC *aprec= &aprec_, *iprec= &iprec_;
     MYSQL_FIELD *field= mysql_fetch_field_direct(result,nSrcCol);
     MYSQL_ROW   row_data= result->data_cursor->data + nSrcCol;
     NET         *net=&stmt->dbc->mysql.net;
-    SQLCHAR     *to= net->buff;
+    unsigned char *to= net->buff;
     SQLLEN      length;
 
+    desc_rec_init_apd(aprec);
+    desc_rec_init_ipd(iprec);
+
     /* Copy row buffer data to statement */
-    param.used= 1;
-    param.SqlType= get_sql_data_type(stmt, field, 0);
-    param.CType= SQL_C_CHAR;
+    iprec->concise_type= get_sql_data_type(stmt, field, 0);
+    aprec->concise_type= SQL_C_CHAR;
 
     if ( row_data && *row_data )
     {
-        param.buffer= *row_data;
+        aprec->data_ptr= (SQLPOINTER) *row_data;
         length= strlen(*row_data);
 
-        param.actual_len= &length;
+        aprec->octet_length_ptr= &length;
+        aprec->indicator_ptr= &length;
 
-        if ( copy_field_data(stmt,&param,&net,&to) != SQL_SUCCESS )
-            return 1;
+        if (!SQL_SUCCEEDED(insert_param(stmt, (char **) &to, stmt->apd,
+                                        aprec, iprec, 0)))
+          return 1;
+        if (!(to= (unsigned char *) add_to_buffer(net, (char *) to, " AND ", 5)))
+          return set_error(stmt, MYERR_S1001, NULL, 4001);
 
         length= (uint) ((char *)to - (char*) net->buff);
         dynstr_append_mem(dynQuery, (char*) net->buff, length);
@@ -522,23 +516,6 @@ static my_bool insert_field(STMT FAR *stmt, MYSQL_RES *result,
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : checks for the float comparision in where clause
-*/
-static my_bool if_float_field(STMT FAR *stmt, MYSQL_FIELD *field)
-{
-    if ( field->type == MYSQL_TYPE_FLOAT || field->type == MYSQL_TYPE_DOUBLE ||
-         field->type == MYSQL_TYPE_DECIMAL )
-    {
-        set_error(stmt,MYERR_S1000,
-                  "Invalid use of floating point comparision in positioned operations",0);
-        return 1;
-    }
-    return 0;
-}
-
-
 /*
   @type    : myodbc3 internal
   @purpose : checks for the existance of pk columns in the resultset,
@@ -592,8 +569,7 @@ static SQLRETURN insert_pk_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
   @purpose : generate a WHERE clause based on the fields in the result set
 */
 
-static SQLRETURN append_all_fields(STMT FAR *stmt,
-                                   DYNAMIC_STRING *dynQuery)
+static SQLRETURN append_all_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
 {
   MYSQL_RES    *result= stmt->result;
   MYSQL_RES    *presultAllColumns;
@@ -649,8 +625,12 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
       We also can't handle floating-point fields because their comparison
       is inexact.
     */
-    if (if_float_field(stmt, table_field))
+    if (table_field->type == MYSQL_TYPE_FLOAT ||
+        table_field->type == MYSQL_TYPE_DOUBLE ||
+        table_field->type == MYSQL_TYPE_DECIMAL)
     {
+      set_error(stmt,MYERR_S1000,
+                "Invalid use of floating point comparision in positioned operations",0);
       mysql_free_result(presultAllColumns);
       return SQL_ERROR;
     }
@@ -689,7 +669,7 @@ static SQLRETURN append_all_fields(STMT FAR *stmt,
 }
 
 /*
-  @type	  : myodbc3 internal
+  @type    : myodbc3 internal
   @purpose : build the where clause
 */
 
@@ -728,7 +708,7 @@ static SQLRETURN build_where_clause( STMT FAR *       pStmt,
         char buff[32];
 
         sprintf( buff, " LIMIT %lu",
-                 (unsigned long)pStmt->stmt_options.rows_in_set );
+                 (unsigned long)pStmt->ard->array_size );
         dynstr_append( dynQuery, buff );
     }
     else
@@ -740,44 +720,28 @@ static SQLRETURN build_where_clause( STMT FAR *       pStmt,
 }
 
 
-/*
-  @type	  : myodbc3 internal
-  @purpose : if input param buffers exist, copy them to new
-  statement
-*/
-
-static void copy_input_param(STMT FAR *stmt,STMT FAR *stmtNew,
-                             SQLUINTEGER pcount)
-{
-    while ( pcount-- )
-    {
-        PARAM_BIND *param= dynamic_element(&stmt->params,pcount,PARAM_BIND*);
-        PARAM_BIND *paramNew= dynamic_element(&stmtNew->params,pcount,PARAM_BIND*);
-        param->pos_in_query= paramNew->pos_in_query;
-        set_dynamic(&stmtNew->params, (DYNAMIC_ELEMENT)param, pcount);
-    }
-}
-
-
 /*
   @type    : myodbc3 internal
   @purpose : set clause building..
 */
 
-static SQLRETURN build_set_clause(STMT FAR *stmt, SQLUINTEGER irow,
+static SQLRETURN build_set_clause(STMT FAR *stmt, SQLULEN irow,
                                   DYNAMIC_STRING *dynQuery)
 {
-    PARAM_BIND    param;
-    SQLLEN        length;
+    DESCREC aprec_, iprec_;
+    DESCREC *aprec= &aprec_, *iprec= &iprec_;
+    SQLLEN        length= 0;
     uint          ncol, ignore_count= 0;
     MYSQL_FIELD *field;
     MYSQL_RES   *result= stmt->result;
-    BIND        *bind;
     NET         *net=&stmt->dbc->mysql.net;
-    SQLLEN      *pcbValue;
+    DESCREC *arrec, *irrec;
 
     dynstr_append_mem(dynQuery," SET ",5);
 
+    desc_rec_init_apd(aprec);
+    desc_rec_init_ipd(iprec);
+
     /*
       To make sure, it points to correct row in the
       current rowset..
@@ -785,59 +749,78 @@ static SQLRETURN build_set_clause(STMT FAR *stmt, SQLUINTEGER irow,
     irow= irow ? irow-1: 0;
     for ( ncol= 0; ncol < stmt->result->field_count; ncol++ )
     {
+        SQLLEN *pcbValue;
         SQLCHAR *to= net->buff;
         field= mysql_fetch_field_direct(result,ncol);
-        bind= stmt->bind+ncol;
+        arrec= desc_get_rec(stmt->ard, ncol, FALSE);
+        irrec= desc_get_rec(stmt->ird, ncol, FALSE);
+        assert(irrec);
+        assert(irrec->row.field);
 
-        if (!stmt->bind || (bind && !bind->field))
+        if (stmt->setpos_apd)
+          aprec= desc_get_rec(stmt->setpos_apd, ncol, FALSE);
+
+        if (!arrec || !ARD_IS_BOUND(arrec) || !irrec->row.field)
         {
           ignore_count++;
           continue;
         }
-        pcbValue= bind->pcbValue ? bind->pcbValue + irow : 0;
-        if ( pcbValue )
+
+        if ( arrec->octet_length_ptr )
         {
+            pcbValue= ptr_offset_adjust(arrec->octet_length_ptr,
+                                        stmt->ard->bind_offset_ptr,
+                                        stmt->ard->bind_type,
+                                        sizeof(SQLLEN), irow);
             /*
-          If the pcbValue is SQL_COLUMN_IGNORE, then ignore the
-          column in the SET clause
+              If the pcbValue is SQL_COLUMN_IGNORE, then ignore the
+              column in the SET clause
             */
             if ( *pcbValue == SQL_COLUMN_IGNORE )
             {
                 ignore_count++;
                 continue;
             }
-            /*
-          Take care of SQL_NTS in pcbValue
-            */
-            else if ( *pcbValue == SQL_NTS )
-                length= SQL_NTS;
-            else
-                length= *pcbValue;
+            length= *pcbValue;
         }
         else
-            length= SQL_NTS;
+        {
+            /* set SQL_NTS only if its a string */
+            switch (arrec->concise_type)
+            {
+                case SQL_CHAR:
+                case SQL_VARCHAR:
+                case SQL_LONGVARCHAR:
+                    length= SQL_NTS;
+                    break;
+            }
+        }
 
-        /* TODO : handle ..SQL_DATA_AT_EXEC here....*/
-
-        dynstr_append_quoted_name(dynQuery,field->name);
+        dynstr_append_quoted_name(dynQuery,field->org_name);
         dynstr_append_mem(dynQuery,"=",1);
 
-        param.used= 1;
-        param.SqlType= get_sql_data_type(stmt, field, NULL);
-        param.CType= bind->fCType;
-        param.buffer= ((char *)bind->rgbValue)+irow*bind->cbValueMax;
-        param.ValueMax= bind->cbValueMax;
-        /*
-            Check when SQL_LEN_DATA_AT_EXEC() macro was used instead of data length
-        */
+        iprec->concise_type= get_sql_data_type(stmt, field, NULL);
+        aprec->concise_type= arrec->concise_type;
+	/* copy prec and scale - needed for SQL_NUMERIC values */
+	iprec->precision= arrec->precision;
+	iprec->scale= arrec->scale;
+        if (stmt->dae_type && aprec->par.is_dae)
+          aprec->data_ptr= aprec->par.value;
+        else
+          aprec->data_ptr= ptr_offset_adjust(arrec->data_ptr,
+                                             stmt->ard->bind_offset_ptr,
+                                             stmt->ard->bind_type,
+                                             bind_length(arrec->concise_type,
+                                                         arrec->octet_length),
+                                             irow);
+        aprec->octet_length= arrec->octet_length;
         if (length == SQL_NTS)
-            length= strlen(param.buffer);
-        else if (length <= SQL_LEN_DATA_AT_EXEC_OFFSET)
-            length= -(length - SQL_LEN_DATA_AT_EXEC_OFFSET);
+            length= strlen(aprec->data_ptr);
 
-        param.actual_len= &length;
+        aprec->octet_length_ptr= &length;
+        aprec->indicator_ptr= &length;
 
-        if ( copy_rowdata(stmt,param,&net,&to) != SQL_SUCCESS )
+        if ( copy_rowdata(stmt,aprec,iprec,&net,&to) != SQL_SUCCESS )
             return(SQL_ERROR);
 
         length= (uint) ((char *)to - (char*) net->buff);
@@ -888,13 +871,13 @@ SQLRETURN my_pos_update( STMT FAR *         pStmtCursor,
                          SQLUSMALLINT       nRow, 
                          DYNAMIC_STRING *   dynQuery )
 {
-    SQLRETURN   nReturn;
+    SQLRETURN   rc;
     SQLHSTMT    hStmtTemp;
     STMT FAR  * pStmtTemp;
 
-    nReturn = build_where_clause( pStmtCursor, dynQuery, nRow );
-    if ( !SQL_SUCCEEDED( nReturn ) )
-        return nReturn;
+    rc = build_where_clause( pStmtCursor, dynQuery, nRow );
+    if ( !SQL_SUCCEEDED( rc ) )
+        return rc;
 
     /*
       Prepare and check if parameters exists in set clause..
@@ -907,34 +890,43 @@ SQLRETURN my_pos_update( STMT FAR *         pStmtCursor,
 
     pStmtTemp = (STMT FAR*)hStmtTemp;
 
-    if ( my_SQLPrepare( pStmtTemp, (SQLCHAR FAR*)dynQuery->str, dynQuery->length ) != SQL_SUCCESS )
+    if (my_SQLPrepare(pStmtTemp, (SQLCHAR *)dynQuery->str, dynQuery->length,
+                      FALSE) != SQL_SUCCESS)
     {
         my_SQLFreeStmt( pStmtTemp, SQL_DROP );
         return set_stmt_error( pStmt, "HY000", "my_SQLPrepare() failed.", 0 );
     }
     if ( pStmtTemp->param_count )      /* SET clause has parameters */
-        copy_input_param( pStmt, pStmtTemp, pStmtTemp->param_count );
+    {
+        if (!SQL_SUCCEEDED(rc= stmt_SQLCopyDesc(pStmt, pStmt->apd,
+                                                pStmtTemp->apd)))
+          return rc;
+        if (!SQL_SUCCEEDED(rc= stmt_SQLCopyDesc(pStmt, pStmt->ipd,
+                                                pStmtTemp->ipd)))
+          return rc;
+    }
 
-    nReturn = my_SQLExecute( pStmtTemp );
-    if ( SQL_SUCCEEDED( nReturn ) )
+    rc = my_SQLExecute( pStmtTemp );
+    if ( SQL_SUCCEEDED( rc ) )
     {
         pStmt->affected_rows = mysql_affected_rows( &pStmtTemp->dbc->mysql );
-        nReturn = update_status( pStmt, SQL_ROW_UPDATED );
+        rc = update_status( pStmt, SQL_ROW_UPDATED );
     }
-    else if (nReturn == SQL_NEED_DATA)
+    else if (rc == SQL_NEED_DATA)
     {
       /*
         Re-prepare the statement, which will leave us with a prepared
         statement that is a non-positioned update.
       */
-      if (my_SQLPrepare(pStmt, (SQLCHAR *)dynQuery->str, dynQuery->length) !=
-          SQL_SUCCESS)
+      if (my_SQLPrepare(pStmt, (SQLCHAR *)dynQuery->str, dynQuery->length,
+                        FALSE) != SQL_SUCCESS)
         return SQL_ERROR;
+      pStmt->dae_type= DAE_NORMAL;
     }
 
     my_SQLFreeStmt( pStmtTemp, SQL_DROP );
 
-    return nReturn;
+    return rc;
 }
 
 
@@ -988,6 +980,10 @@ static SQLRETURN setpos_delete(STMT FAR *stmt, SQLUSMALLINT irow,
     if ( nReturn == SQL_SUCCESS )
         nReturn= update_setpos_status(stmt,irow,affected_rows,SQL_ROW_DELETED);
 
+    /* fix-up so fetching next rowset is correct */
+    if (if_dynamic_cursor(stmt))
+      stmt->rows_found_in_set-= (uint) affected_rows;
+
     return nReturn;
 }
 
@@ -1087,24 +1083,27 @@ static SQLRETURN setpos_update(STMT FAR *stmt, SQLUSMALLINT irow,
     \retval SQL_SUCCESS     Success!
 */
 
-static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING *ext_query )
+static SQLRETURN batch_insert( STMT FAR *stmt, SQLULEN irow, DYNAMIC_STRING *ext_query )
 {
-    MYSQL_RES    *result= stmt->result;     /* result set we are working with                               */
-    SQLUINTEGER  insert_count= 1;           /* num rows to insert - will be real value when row is 0 (all)  */
-    SQLUINTEGER  count= 0;                  /* current row                                                  */
+    MYSQL_RES    *result= stmt->result;     /* result set we are working with */
+    SQLULEN      insert_count= 1;           /* num rows to insert - will be real value when row is 0 (all)  */
+    SQLULEN      count= 0;                  /* current row */
     SQLLEN       length;
     NET         *net= &stmt->dbc->mysql.net;
     SQLUSMALLINT ncol;
     SQLCHAR      *to;
     ulong        query_length= 0;           /* our original query len so we can reset pos if break_insert   */
-    my_bool      break_insert= FALSE;       /* true if we are to exceed max data size for transmission      
-                                               but this seems to be misused                                 */
-    PARAM_BIND   param;
+    my_bool      break_insert= FALSE;       /* true if we are to exceed max data size for transmission
+                                               but this seems to be misused */
+    DESCREC aprec_, iprec_;
+    DESCREC *aprec= &aprec_, *iprec= &iprec_;
+
+    desc_rec_init_ipd(iprec);
 
     /* determine the number of rows to insert when irow = 0 */
-    if ( !irow && stmt->stmt_options.rows_in_set > 1 ) /* batch wise */
+    if ( !irow && stmt->ard->array_size > 1 ) /* batch wise */
     {
-        insert_count= stmt->stmt_options.rows_in_set;
+        insert_count= stmt->ard->array_size;
         query_length= ext_query->length;
     }
 
@@ -1128,41 +1127,53 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
             for ( ncol= 0; ncol < result->field_count; ncol++ )
             {
                 MYSQL_FIELD *field= mysql_fetch_field_direct(result,ncol);
-                BIND        *bind= stmt->bind+ncol;
-                SQLINTEGER   binding_offset= 0;
-                SQLUINTEGER  element_size= 0;
-                SQLLEN       ind_or_len;
+                DESCREC     *arrec;
+                SQLLEN       ind_or_len= 0;
 
-                if (stmt->stmt_options.bind_type != SQL_BIND_BY_COLUMN &&
-                    stmt->stmt_options.bind_offset)
-                  binding_offset= *(stmt->stmt_options.bind_offset);
-
-                if (stmt->stmt_options.bind_type != SQL_BIND_BY_COLUMN)
-                  element_size= stmt->stmt_options.bind_type;
-
-                if (bind->pcbValue)
-                  ind_or_len= *(SQLLEN *)((char *)bind->pcbValue +
-                                          binding_offset +
-                                          count * (element_size ?
-                                                   element_size :
-                                                   sizeof(SQLLEN)));
+                arrec= desc_get_rec(stmt->ard, ncol, FALSE);
+                /* if there's a separate APD for this (dae), use it */
+                if (stmt->setpos_apd)
+                  aprec= desc_get_rec(stmt->setpos_apd, ncol, FALSE);
                 else
-                  ind_or_len= bind->cbValueMax;
+                  desc_rec_init_apd(aprec);
+                
+                if (arrec)
+                {
+                  if (aprec->par.is_dae)
+                    ind_or_len= aprec->par.value_length;
+                  else if (arrec->octet_length_ptr)
+                    ind_or_len= *(SQLLEN *)
+                                ptr_offset_adjust(arrec->octet_length_ptr,
+                                                  stmt->ard->bind_offset_ptr,
+                                                  stmt->ard->bind_type,
+                                                  sizeof(SQLLEN), count);
+                  else
+                    ind_or_len= arrec->octet_length;
 
-                param.SqlType= get_sql_data_type(stmt, field, NULL);
-                param.CType = bind->fCType;
-                param.buffer= ((char *)bind->rgbValue +
-                               binding_offset +
-                               count * (element_size ?
-                                        element_size :
-                                        bind_length(bind->fCType,
-                                                    bind->cbValueMax)));
+                  iprec->concise_type= get_sql_data_type(stmt, field, NULL);
+                  aprec->concise_type= arrec->concise_type;
+                  /* copy prec and scale - needed for SQL_NUMERIC values */
+                  iprec->precision= arrec->precision;
+                  iprec->scale= arrec->scale;
+
+                  if (stmt->dae_type && aprec->par.is_dae)
+                    /* arrays or offsets are not supported for data-at-exec */
+                    aprec->data_ptr= aprec->par.value;
+                  else
+                    aprec->data_ptr= ptr_offset_adjust(arrec->data_ptr,
+                                                       stmt->ard->bind_offset_ptr,
+                                                       stmt->ard->bind_type,
+                                                       bind_length(arrec->concise_type,
+                                                                   arrec->octet_length),
+                                                       count);
+                }
 
                 switch (ind_or_len) {
                 case SQL_NTS:
-                  if (param.buffer)
-                    length= strlen(param.buffer);
+                  if (aprec->data_ptr)
+                    length= strlen(aprec->data_ptr);
                   break;
+
                 /*
                   We pass through SQL_COLUMN_IGNORE and SQL_NULL_DATA,
                   because the insert_data() that is eventually called knows
@@ -1174,9 +1185,10 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
                   length= ind_or_len;
                 }
 
-                param.actual_len= &length;
+                aprec->octet_length_ptr= &length;
+                aprec->indicator_ptr= &length;
 
-                if (copy_rowdata(stmt, param, &net, &to) != SQL_SUCCESS)
+                if (copy_rowdata(stmt, aprec, iprec, &net, &to) != SQL_SUCCESS)
                   return SQL_ERROR;
 
             } /* END OF for (ncol= 0; ncol < result->field_count; ncol++) */
@@ -1190,8 +1202,7 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
               We have a limited capacity to shove data across the wire, but
               we handle this by sending in multiple calls to exec_stmt_query()
             */
-            if ((size_t)ext_query->length + (size_t)length >=
-                (size_t)net_buffer_length)
+            if (ext_query->length + length >= (SQLLEN) net_buffer_length)
             {
                 break_insert= TRUE;
                 break;
@@ -1210,10 +1221,10 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
     stmt->affected_rows= stmt->dbc->mysql.affected_rows= insert_count;
 
     /* update row status pointer(s) */
-    if (stmt->stmt_options.rowStatusPtr)
+    if (stmt->ird->array_status_ptr)
     {
       for (count= insert_count; count--; )
-        stmt->stmt_options.rowStatusPtr[count]= SQL_ROW_ADDED;
+        stmt->ird->array_status_ptr[count]= SQL_ROW_ADDED;
     }
     if (stmt->stmt_options.rowStatusPtr_ex)
     {
@@ -1225,6 +1236,48 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
 }
 
 
+/*
+  Setup a data-at-execution for SQLSetPos() on the current
+  statement.
+*/
+static SQLRETURN setpos_dae_check_and_init(STMT *stmt, SQLSETPOSIROW irow,
+                                           SQLSMALLINT fLock, char dae_type)
+{
+  int dae_rec;
+  SQLRETURN rc;
+  
+  /*
+    If this statement hasn't already had the dae params set,
+    check if there are any and begin the SQLParamData() sequence.
+  */
+  if (stmt->dae_type != DAE_SETPOS_DONE &&
+      (dae_rec= desc_find_dae_rec(stmt->ard)) > -1)
+  {
+    if (!irow && stmt->ard->array_size > 1)
+      return set_stmt_error(stmt, "HYC00", "Multiple row insert "
+                            "with data at execution not supported",
+                            0);
+    
+    /* create APD, and copy ARD to it */
+    stmt->setpos_apd= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
+                                 DESC_APP, DESC_PARAM);
+    if (!stmt->setpos_apd)
+      return set_stmt_error(stmt, "S1001", "Not enough memory",
+                            4001);
+    if(rc= stmt_SQLCopyDesc(stmt, stmt->ard, stmt->setpos_apd))
+      return rc;
+                  
+    stmt->current_param= dae_rec;
+    stmt->dae_type= dae_type;
+    stmt->setpos_row= irow;
+    stmt->setpos_lock= fLock;
+    return SQL_NEED_DATA;
+  }
+
+  return SQL_SUCCESS;
+}
+
+
 /*!
     \brief  Shadow function for SQLSetPos.
     
@@ -1245,11 +1298,13 @@ static SQLRETURN batch_insert( STMT FAR *stmt, SQLUSMALLINT irow, DYNAMIC_STRING
 static const char *alloc_error= "Driver Failed to set the internal dynamic result";
 
 
-static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSMALLINT fOption, SQLUSMALLINT fLock )
+SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
+                               SQLUSMALLINT fOption, SQLUSMALLINT fLock)
 {
     STMT FAR  *stmt= (STMT FAR*) hstmt;
     SQLRETURN sqlRet= SQL_SUCCESS;
     MYSQL_RES *result= stmt->result;
+    SQLRETURN rc;
 
     CLEAR_STMT_ERROR(stmt);
 
@@ -1286,11 +1341,9 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 stmt->cursor_row= (long)(stmt->current_row+irow);
                 mysql_data_seek(stmt->result,(my_ulonglong)stmt->cursor_row);
                 stmt->current_values= mysql_fetch_row(stmt->result);
-                stmt->last_getdata_col= (uint)  ~0;; /* reset */
+                reset_getdata_position(stmt);
                 if ( stmt->fix_fields )
                     stmt->current_values= (*stmt->fix_fields)(stmt,stmt->current_values);
-                else
-                    stmt->result_lengths= mysql_fetch_lengths(stmt->result);
                 /*
                  The call to mysql_fetch_row() moved stmt->result's internal
                  cursor, but we don't want that. We seek back to this row
@@ -1329,8 +1382,13 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                     return set_error(stmt,MYERR_S1107,NULL,0);
 
                 /* IF dynamic cursor THEN rerun query to refresh resultset */
-                if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
+                if (!stmt->dae_type && if_dynamic_cursor(stmt) &&
+                    set_dynamic_result(stmt))
+                  return set_error(stmt,MYERR_S1000, alloc_error, 0);
+
+                if (rc= setpos_dae_check_and_init(stmt, irow, fLock,
+                                                  DAE_SETPOS_UPDATE))
+                  return rc;
 
                 if ( init_dynamic_string(&dynQuery, "UPDATE ", 1024, 1024) )
                     return set_error(stmt,MYERR_S1001,NULL,4001);
@@ -1346,17 +1404,29 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 DYNAMIC_STRING  dynQuery;
                 SQLUSMALLINT    nCol        = 0;
 
-                if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
+                if (!stmt->dae_type && if_dynamic_cursor(stmt) &&
+                    set_dynamic_result(stmt))
+                  return set_error(stmt,MYERR_S1000, alloc_error, 0);
                 result= stmt->result;
 
                 if ( !(table_name= find_used_table(stmt)) )
                     return SQL_ERROR;
 
+                if (rc= setpos_dae_check_and_init(stmt, irow, fLock,
+                                                  DAE_SETPOS_INSERT))
+                  return rc;
+
                 if ( init_dynamic_string(&dynQuery, "INSERT INTO ", 1024,1024) )
                     return set_stmt_error(stmt, "S1001", "Not enough memory",
                                           4001);
 
+				/* Append the table's DB name if exists */
+				if(result->fields && result->fields[0].db_length)
+				{
+					dynstr_append_quoted_name(&dynQuery, result->fields[0].db);
+					dynstr_append_mem(&dynQuery,".",1);
+				}
+
                 dynstr_append_quoted_name(&dynQuery,table_name);
                 dynstr_append_mem(&dynQuery,"(",1);
 
@@ -1364,7 +1434,7 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                 for (nCol= 0; nCol < result->field_count; nCol++)
                 {
                     MYSQL_FIELD *field= mysql_fetch_field_direct(result, nCol);
-                    dynstr_append_quoted_name(&dynQuery, field->name);
+                    dynstr_append_quoted_name(&dynQuery, field->org_name);
                     dynstr_append_mem(&dynQuery, ",", 1);
                 }
                 dynQuery.length--;        /* Remove last ',' */
@@ -1385,10 +1455,10 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
                   buffers
                 */
                 sqlRet= my_SQLExtendedFetch(hstmt, SQL_FETCH_ABSOLUTE, irow,
-                                            stmt->stmt_options.rowsFetchedPtr,
+                                            stmt->ird->rows_processed_ptr,
                                             stmt->stmt_options.rowStatusPtr_ex ?
                                             stmt->stmt_options.rowStatusPtr_ex :
-                                            stmt->stmt_options.rowStatusPtr, 0);
+                                            stmt->ird->array_status_ptr, 0);
                 break;
             }
 
@@ -1399,78 +1469,51 @@ static SQLRETURN SQL_API my_SQLSetPos( SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSM
 }
 
 
-/*
-  @type    : ODBC 1.0 API
-  @purpose : associates a cursor name with an active statement.
-  If an application does not call SQLSetCursorName, the driver
-  generates cursor names as needed for SQL statement processing
-*/
-
-SQLRETURN SQL_API SQLSetCursorName(SQLHSTMT hstmt, SQLCHAR *szCursor,
-                                   SQLSMALLINT cbCursor)
+SQLRETURN SQL_API
+MySQLSetCursorName(SQLHSTMT hstmt, SQLCHAR *name, SQLSMALLINT len)
 {
-    STMT FAR *stmt= (STMT FAR*) hstmt;
+  STMT *stmt= (STMT *) hstmt;
 
-    CLEAR_STMT_ERROR(stmt);
+  CLEAR_STMT_ERROR(stmt);
 
-    if ( !szCursor )
-        return set_error(stmt,MYERR_S1009,NULL,0);
+  if (!name)
+    return set_error(stmt, MYERR_S1009, NULL, 0);
 
-    if ( cbCursor == SQL_NTS )
-        cbCursor= (SQLSMALLINT) strlen((char*) szCursor);
+  if (len == SQL_NTS)
+    len= strlen((char *)name);
 
-    if ( cbCursor < 0 )
-        return set_error(stmt,MYERR_S1090,NULL,0);
+  if (len < 0)
+    return set_error(stmt, MYERR_S1009, NULL, 0);
 
-    if ( (cbCursor == 0) ||
-         (cbCursor > MYSQL_MAX_CURSOR_LEN) ||
-         (myodbc_casecmp((char*) szCursor, "SQLCUR", 6) == 0)  ||
-         (myodbc_casecmp((char*) szCursor, "SQL_CUR", 7) == 0) )
-        return set_error(stmt,MYERR_34000,NULL,0);
+  /** @todo use charset-aware casecmp */
+  if (len == 0 ||
+      len > MYSQL_MAX_CURSOR_LEN ||
+      myodbc_casecmp((char *)name, "SQLCUR", 6) == 0 ||
+      myodbc_casecmp((char *)name, "SQL_CUR", 7) == 0)
+    return set_error(stmt, MYERR_34000, NULL, 0);
 
-    x_free(stmt->cursor.name);
-    stmt->cursor.name= dupp_str((char*) szCursor,cbCursor);
-    return SQL_SUCCESS;
+  x_free(stmt->cursor.name);
+  stmt->cursor.name= dupp_str((char*)name, len);
+
+  return SQL_SUCCESS;
 }
 
 
-/*
-  @type    : ODBC 1.0 API
-  @purpose : returns the cursor name associated with a specified statement
+/**
+  Get the current cursor name, generating one if necessary.
+
+  @param[in]   hstmt  Statement handle
+
+  @return  Pointer to cursor name (terminated with '\0')
 */
-
-SQLRETURN SQL_API SQLGetCursorName(SQLHSTMT hstmt, SQLCHAR FAR *szCursor,
-                                   SQLSMALLINT cbCursorMax,
-                                   SQLSMALLINT FAR *pcbCursor)
+SQLCHAR *MySQLGetCursorName(HSTMT hstmt)
 {
-    STMT FAR    *stmt= (STMT FAR*) hstmt;
-    SQLINTEGER  nLength;
-    SQLSMALLINT nDummyLength;
+  STMT *stmt= (STMT *)hstmt;
 
-    CLEAR_STMT_ERROR(stmt);
+  if (!stmt->cursor.name)
+    set_dynamic_cursor_name(stmt);
 
-    if ( cbCursorMax < 0 )
-        return set_error(stmt,MYERR_S1090,NULL,0);
-
-    if ( !pcbCursor )
-        pcbCursor= &nDummyLength;
-
-    if ( cbCursorMax )
-        cbCursorMax-= sizeof(char);
-
-    if ( !stmt->cursor.name )
-        set_dynamic_cursor_name(stmt);
-
-    *pcbCursor= strlen(stmt->cursor.name);
-    if ( szCursor && cbCursorMax > 0 )
-        strmake((char*) szCursor, stmt->cursor.name, cbCursorMax);
-
-    nLength= myodbc_min(*pcbCursor , cbCursorMax);
-
-    if ( nLength != *pcbCursor )
-        return set_error(stmt,MYERR_01004,NULL,0);
-
-    return SQL_SUCCESS;
+  return (SQLCHAR *)stmt->cursor.name;
 }
 
 
