@@ -23,7 +23,7 @@
 /***************************************************************************
  * ERROR.H								   *
  *									   *
- * @description: Definations for error handling				   *
+ * @description: Definitions for error handling				   *
  *									   *
  * @author     : MySQL AB(monty@mysql.com, venu@mysql.com)		   *
  * @date       : 2001-Aug-15						   *
@@ -43,25 +43,39 @@
 /*
   myodbc3 error prefix
 */
-#define MYODBC3_ERROR_PREFIX	 "[MySQL][ODBC 3.51 Driver]"
+#define MYODBC3_ERROR_PREFIX	 "[MySQL][ODBC 5.1 Driver]"
 #define MYODBC_ERROR_CODE_START  500
 
 /*
   error handler structure
 */
 typedef struct tagERROR {
+  SQLRETURN   retcode;
+  char        current;
 
   char	   sqlstate[6];
   char	   message[SQL_MAX_MESSAGE_LENGTH+1];
   SQLINTEGER  native_error;
-  SQLRETURN   retcode;
 
 } MYERROR;
 
+#define CLEAR_ERROR(error) do {\
+  error.message[0]= '\0'; \
+  error.current= 0; \
+} while (0)
 
-#define CLEAR_ENV_ERROR(env)   ((ENV FAR *)env)->error.message[0]='\0'
-#define CLEAR_DBC_ERROR(dbc)   ((DBC FAR *)dbc)->error.message[0]='\0'
-#define CLEAR_STMT_ERROR(stmt) ((STMT FAR *)stmt)->error.message[0]='\0'
+#define CLEAR_ENV_ERROR(env)   CLEAR_ERROR(((ENV *)env)->error)
+#define CLEAR_DBC_ERROR(dbc)   CLEAR_ERROR(((DBC *)dbc)->error)
+#define CLEAR_STMT_ERROR(stmt) CLEAR_ERROR(((STMT *)stmt)->error)
+#define CLEAR_DESC_ERROR(desc) CLEAR_ERROR(((DESC *)desc)->error)
+
+#define NEXT_ERROR(error) \
+  (error.current ? 2 : (error.current= 1))
+
+#define NEXT_ENV_ERROR(env)   NEXT_ERROR(((ENV *)env)->error)
+#define NEXT_DBC_ERROR(dbc)   NEXT_ERROR(((DBC *)dbc)->error)
+#define NEXT_STMT_ERROR(stmt) NEXT_ERROR(((STMT *)stmt)->error)
+#define NEXT_DESC_ERROR(desc) NEXT_ERROR(((DESC *)desc)->error)
 
 /*
   list of MyODBC3 error codes
@@ -90,12 +104,15 @@ typedef enum myodbc_errid
     MYERR_S1002,
     MYERR_S1003,
     MYERR_S1004,
+    MYERR_S1007,
     MYERR_S1009,
     MYERR_S1010,
     MYERR_S1011,
     MYERR_S1012,
     MYERR_S1013,
     MYERR_S1015,
+    MYERR_S1016,
+    MYERR_S1017,
     MYERR_S1024,
     MYERR_S1090,
     MYERR_S1091,
@@ -106,6 +123,7 @@ typedef enum myodbc_errid
     MYERR_S1107,
     MYERR_S1109,
     MYERR_S1C00,
+
     MYERR_21S01,
     MYERR_23000,
     MYERR_42000,
