@@ -25,18 +25,7 @@
   @brief Statement execution functions.
 */
 
-/***************************************************************************
- * The following ODBC APIs are implemented in this file:		   *
- *									   *
- *   SQLExecute		 (ISO 92)					   *
- *   SQLExecDirect	 (ISO 92)					   *
- *   SQLParamData	 (ISO 92)					   *
- *   SQLPutData		 (ISO 92)					   *
- *   SQLCancel		 (ISO 92)					   *
- *									   *
- ****************************************************************************/
-
-#include "myodbc3.h"
+#include "driver.h"
 #include <locale.h>
 
 /*
@@ -163,7 +152,7 @@ char *extend_buffer(NET *net, char *to, ulong length)
   @purpose : help function to extend the buffer and copy the data
 */
 
-char *add_to_buffer(NET *net,char *to,char *from,ulong length)
+char *add_to_buffer(NET *net,char *to,const char *from,ulong length)
 {
     if ( !(to= extend_buffer(net,to,length)) )
         return 0;
@@ -179,151 +168,225 @@ char *add_to_buffer(NET *net,char *to,char *from,ulong length)
   @purpose : insert sql params at parameter positions
 */
 
-char *insert_params(STMT FAR *stmt)
+SQLRETURN insert_params(STMT FAR *stmt, char **finalquery)
 {
     char *query= stmt->query,*to;
     uint i,length;
     NET *net;
+    SQLRETURN rc= SQL_SUCCESS;
 
     pthread_mutex_lock(&stmt->dbc->lock);
     net= &stmt->dbc->mysql.net;
     to= (char*) net->buff;
-    if ( !(stmt->dbc->flag & FLAG_NO_LOCALE) )
+    if (!stmt->dbc->ds->dont_use_set_locale)
       setlocale(LC_NUMERIC, "C");  /* force use of '.' as decimal point */
     for ( i= 0; i < stmt->param_count; i++ )
     {
-        PARAM_BIND *param= dynamic_element(&stmt->params,i,PARAM_BIND*);
+        DESCREC *aprec= desc_get_rec(stmt->apd, i, FALSE);
+        DESCREC *iprec= desc_get_rec(stmt->ipd, i, FALSE);
         char *pos;
 
-        if ( !param->used )
+        assert(aprec && iprec);
+
+        if (stmt->dummy_state != ST_DUMMY_PREPARED &&
+            !aprec->par.real_param_done)
         {
-            if ( !(stmt->dbc->flag & FLAG_NO_LOCALE) )
-                setlocale(LC_NUMERIC,default_locale);
-            set_error(stmt,MYERR_07001,NULL,0);
-            pthread_mutex_unlock(&stmt->dbc->lock);
-            return 0;
+            rc= set_error(stmt,MYERR_07001,NULL,0);
+            goto error;
         }
-        pos= param->pos_in_query;
+        get_dynamic(&stmt->param_pos, (void *)&pos, i);
         length= (uint) (pos-query);
         if ( !(to= add_to_buffer(net,to,query,length)) )
-            goto error;
-        query= pos+1;  /* Skipp '?' */
-        if ( !(to= insert_param(stmt->dbc,to,param)) )
+            goto memerror;
+        query= pos+1;  /* Skip '?' */
+        if (!SQL_SUCCEEDED(rc= insert_param(stmt,&to,stmt->apd,aprec,iprec,0)))
             goto error;
     }
     length= (uint) (stmt->query_end - query);
     if ( !(to= add_to_buffer(net,to,query,length+1)) )
-        goto error;
+        goto memerror;
     if ( !(to= (char*) my_memdup((char*) net->buff,
                                  (uint) (to - (char*) net->buff),MYF(0))) )
-    {
-        if ( !(stmt->dbc->flag & FLAG_NO_LOCALE) )
-            setlocale(LC_NUMERIC,default_locale);
-        set_error(stmt,MYERR_S1001,NULL,4001);
-        pthread_mutex_unlock(&stmt->dbc->lock);
-        return 0;
-    }
+        goto memerror;
 
-    if ( stmt->stmt_options.paramProcessedPtr )
-        *stmt->stmt_options.paramProcessedPtr= 1; /* We don't support PARAMSET */
+    /* TODO We don't *yet* support PARAMSET */
+    if ( stmt->apd->rows_processed_ptr )
+        *stmt->apd->rows_processed_ptr= 1;
 
     pthread_mutex_unlock(&stmt->dbc->lock);
-    if ( !(stmt->dbc->flag & FLAG_NO_LOCALE) )
+    if (!stmt->dbc->ds->dont_use_set_locale)
         setlocale(LC_NUMERIC,default_locale);
-    return to;
+    *finalquery= to;
+    return rc;
 
-    error:      /* Too much data */
+memerror:      /* Too much data */
+    rc= set_error(stmt,MYERR_S1001,NULL,4001);
+error:
     pthread_mutex_unlock(&stmt->dbc->lock);
-    if ( !(stmt->dbc->flag & FLAG_NO_LOCALE) )
+    if (!stmt->dbc->ds->dont_use_set_locale)
         setlocale(LC_NUMERIC,default_locale);
-    set_error(stmt,MYERR_S1001,NULL,4001);
-    return 0;
+    return rc;
 }
 
 
 /*
-  @type    : myodbc3 internal
-  @purpose : insert sql param the specified parameter position
-*/
+  Add the value of parameter to a string buffer.
 
-char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
+  @param[in]      mysql
+  @param[in,out]  toptr
+  @param[in]      apd The APD of the current statement
+  @param[in]      aprec The APD record of the parameter
+  @param[in]      iprec The IPD record of the parameter
+*/
+SQLRETURN insert_param(STMT *stmt, char **toptr, DESC* apd,
+                       DESCREC *aprec, DESCREC *iprec, SQLULEN row)
 {
     int length;
-    char buff[128],*data;
-    my_bool convert= 0;
+    char buff[128], *data= NULL;
+    my_bool convert= FALSE, free_data= FALSE;
+    DBC *dbc= stmt->dbc;
     NET *net= &dbc->mysql.net;
+    SQLLEN *octet_length_ptr= NULL;
+    SQLLEN *indicator_ptr= NULL;
+    char *to= *toptr;
 
-    if ( !param->actual_len || *(param->actual_len) == SQL_NTS )
+    if (aprec->octet_length_ptr)
     {
-        if ( (data= param->buffer) )
-        {
-            if ( param->actual_len &&  *(param->actual_len) == SQL_NTS )
-                length= strlen(data);
-            else if ( param->ValueMax )
-            {
-#ifndef strnlen
-                length= strlen(data);
-                /* 
-                  For safety reasons add checking for SQL_SETPARAM_VALUE_MAX (-1)
-                  The negative length can be passed through the deprecated 
-                  function SQLSetParam 
-                */
-                if (param->ValueMax != SQL_SETPARAM_VALUE_MAX && 
-                    length > param->ValueMax)
-                {
-                  length= param->ValueMax;
-                }
-#else
-                length=strnlen(data,param->ValueMax);
-#endif
-            }
-            else
-                length=strlen(data);
-        }
-        else
-        {
-            length= 0;     /* This is actually an error */
-        }
+      octet_length_ptr= ptr_offset_adjust(aprec->octet_length_ptr,
+                                          apd->bind_offset_ptr,
+                                          apd->bind_type,
+                                          sizeof(SQLLEN), row);
+      length= *octet_length_ptr;
     }
-    else if ( *(param->actual_len) == SQL_NULL_DATA )
+    indicator_ptr= ptr_offset_adjust(aprec->indicator_ptr,
+                                     apd->bind_offset_ptr,
+                                     apd->bind_type,
+                                     sizeof(SQLLEN), row);
+
+    if (aprec->data_ptr)
     {
-        return add_to_buffer(net,to,"NULL",4);
+      SQLINTEGER default_size= bind_length(aprec->concise_type,
+                                           aprec->octet_length);
+      data= ptr_offset_adjust(aprec->data_ptr, apd->bind_offset_ptr,
+                              apd->bind_type, default_size, row);
+    }
+
+    if (indicator_ptr && *indicator_ptr == SQL_NULL_DATA)
+    {
+      *toptr= add_to_buffer(net,*toptr,"NULL",4);
+      return SQL_SUCCESS;
+    }
+    else if (!octet_length_ptr || *octet_length_ptr == SQL_NTS)
+    {
+      if (data)
+      {
+        if (aprec->concise_type == SQL_C_WCHAR)
+          length= sqlwcharlen((SQLWCHAR *)data) * sizeof(SQLWCHAR);
+        else /* TODO this is stupid, check condition above, shouldn't we be checking only octet_length, not ptr? */
+          length= strlen(data);
+
+        if (!octet_length_ptr && aprec->octet_length > 0 &&
+            aprec->octet_length != SQL_SETPARAM_VALUE_MAX)
+          length= myodbc_min(length, aprec->octet_length);
+      }
+      else
+      {
+        length= 0;     /* TODO? This is actually an error */
+      }
     }
     /*
       We may see SQL_COLUMN_IGNORE from bulk INSERT operations, where we
       may have been told to ignore a column in one particular row. So we
       try to insert DEFAULT, or NULL for really old servers.
+
       In case there are less parameters than result columns we have to
       insert NULL or DEFAULT.
     */
-    else if (*(param->actual_len) == SQL_COLUMN_IGNORE ||
-             (*(param->actual_len) == 0 && param->CType == 0 && 
-              param->buffer == NULL))
+    else if (*octet_length_ptr == SQL_COLUMN_IGNORE ||
+             /* empty values mean it's an unbound column */
+             (*octet_length_ptr == 0 &&
+              aprec->concise_type == SQL_C_DEFAULT &&
+              aprec->par.value == NULL))
     {
       if (is_minimum_version(dbc->mysql.server_version, "4.0.3", 5))
-        return add_to_buffer(net,to,"DEFAULT",7);
+        *toptr= add_to_buffer(net,*toptr,"DEFAULT",7);
       else
-        return add_to_buffer(net,to,"NULL",4);
+        *toptr= add_to_buffer(net,*toptr,"NULL",4);
+      return SQL_SUCCESS;
     }
-    else if ( *param->actual_len == SQL_DATA_AT_EXEC ||
-              *param->actual_len <= SQL_LEN_DATA_AT_EXEC_OFFSET )
+    else if (IS_DATA_AT_EXEC(octet_length_ptr))
     {
-        length= param->value_length;
-        if ( !(data= param->value) )
-            return add_to_buffer(net,to,"NULL",4);
-    }
-    else
-    {
-        data= param->buffer;
-        length= *param->actual_len;
+        length= aprec->par.value_length;
+        if ( !(data= aprec->par.value) )
+        {
+            *toptr= add_to_buffer(net,*toptr,"NULL",4);
+            return SQL_SUCCESS;
+        }
     }
 
-    switch ( param->CType )
+    switch ( aprec->concise_type )
     {
         case SQL_C_BINARY:
         case SQL_C_CHAR:
             convert= 1;
             break;
+
+        case SQL_C_WCHAR:
+          {
+            /* Convert SQL_C_WCHAR (utf-16 or utf-32) to utf-8. */
+            char *to;
+            int i= 0;
+            int utf8len, has_utf8_maxlen4= 0;
+
+            /* length is in bytes, we want chars */
+            length= length / sizeof(SQLWCHAR);
+
+            /* Use buff if it is big enough, otherwise alloc some space. */
+            if (sizeof(buff) >= (size_t)length * 4)
+              to= buff;
+            else
+            {
+              if (!(to= (char *)my_malloc(length * 4, MYF(0))))
+                goto memerror;
+              free_data= TRUE;
+            }
+
+            if (sizeof(SQLWCHAR) == 4)
+            {
+              UTF32 *in= (UTF32 *)data;
+              data= to;
+              while (i < length)
+              {
+                to+= (utf8len= utf32toutf8(in[i++], (UTF8 *)to));
+                if (utf8len == 4)
+                  has_utf8_maxlen4= 1;
+              }
+            }
+            else
+            {
+              UTF16 *in= (UTF16 *)data;
+              data= to;
+              while (i < length)
+              {
+                UTF32 c;
+                i+= utf16toutf32(in + i, &c);
+                to+= (utf8len= utf32toutf8(c, (UTF8 *)to));
+                if (utf8len == 4)
+                  has_utf8_maxlen4= 1;
+              }
+            }
+
+            if (has_utf8_maxlen4 &&
+                !is_minimum_version(dbc->mysql.server_version, "6.0.4", 5))
+              return set_stmt_error(stmt, "HY000",
+                                    "Server does not support 4-byte encoded "
+                                    "UTF8 characters.", 0);
+
+            length= to - data;
+
+            break;
+          }
+
         case SQL_C_BIT:
         case SQL_C_TINYINT:
         case SQL_C_STINYINT:
@@ -361,7 +424,7 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
             data= buff;
             break;
         case SQL_C_FLOAT:
-            if ( param->SqlType != SQL_NUMERIC && param->SqlType != SQL_DECIMAL )
+            if ( iprec->concise_type != SQL_NUMERIC && iprec->concise_type != SQL_DECIMAL )
                 sprintf(buff,"%.17e",*((float*) data));
             else
                 /* We should perpare this data for string comparison */
@@ -369,7 +432,7 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
             length= strlen(data= buff);
             break;
         case SQL_C_DOUBLE:
-            if ( param->SqlType != SQL_NUMERIC && param->SqlType != SQL_DECIMAL )
+            if ( iprec->concise_type != SQL_NUMERIC && iprec->concise_type != SQL_DECIMAL )
                 sprintf(buff,"%.17e",*((double*) data));
             else
                 /* We should perpare this data for string comparison */
@@ -380,7 +443,7 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
         case SQL_C_TYPE_DATE:
             {
                 DATE_STRUCT *date= (DATE_STRUCT*) data;
-                if ((dbc->flag & FLAG_MIN_DATE_TO_ZERO) &&
+                if (dbc->ds->min_date_to_zero &&
                     !date->year && (date->month == date->day == 1))
                   sprintf(buff, "0000-00-00");
                 else
@@ -404,7 +467,7 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
         case SQL_C_TYPE_TIMESTAMP:
             {
                 TIMESTAMP_STRUCT *time= (TIMESTAMP_STRUCT*) data;
-                if ((dbc->flag & FLAG_MIN_DATE_TO_ZERO) &&
+                if (dbc->ds->min_date_to_zero &&
                     !time->year && (time->month == time->day == 1))
                   sprintf(buff, "0000-00-00 %02d:%02d:%02d",
                           time->hour, time->minute, time->second);
@@ -416,16 +479,35 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
                 length= 19;
                 break;
             }
+        case SQL_C_NUMERIC:
+            {
+              int trunc;
+              SQL_NUMERIC_STRUCT *sqlnum= (SQL_NUMERIC_STRUCT *) data;
+              sqlnum_to_str(sqlnum, (SQLCHAR *)(buff + sizeof(buff) - 1),
+                            (SQLCHAR **) &data,
+                            (SQLCHAR) iprec->precision,
+                            (SQLSCHAR) iprec->scale, &trunc);
+              length= strlen(data);
+              /* TODO no way to return an error here? */
+              if (trunc == SQLNUM_TRUNC_FRAC)
+              {/* 01S07 SQL_SUCCESS_WITH_INFO */}
+              else if (trunc == SQLNUM_TRUNC_WHOLE)
+              {/* 22003 SQL_ERROR */
+                return SQL_ERROR;
+              }
+            }
     }
-
-    switch ( param->SqlType )
+    switch ( iprec->concise_type )
     {
         case SQL_DATE:
         case SQL_TYPE_DATE:
         case SQL_TYPE_TIMESTAMP:
         case SQL_TIMESTAMP:
-            if ( data[0] == '{' )       /* Of type {d date } */
-                return add_to_buffer(net,to,data,length);
+            if (data[0] == '{')       /* Of type {d date } */
+            {
+              to= add_to_buffer(net, to, data, length);
+              goto out;
+            }
             /* else treat as a string */
         case SQL_CHAR:
         case SQL_VARCHAR:
@@ -436,16 +518,29 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
         case SQL_WCHAR:
         case SQL_WVARCHAR:
         case SQL_WLONGVARCHAR:
-            break;
-
+            {
+              if (aprec->concise_type == SQL_C_WCHAR &&
+                  dbc->cxn_charset_info->number != UTF8_CHARSET_NUMBER)
+                to= add_to_buffer(net, to, "_utf8", 5);
+              else if (aprec->concise_type != SQL_C_WCHAR &&
+                       dbc->cxn_charset_info->number !=
+                       dbc->ansi_charset_info->number)
+              {
+                to= add_to_buffer(net, to, "_", 1);
+                to= add_to_buffer(net, to, dbc->ansi_charset_info->csname,
+                                  strlen(dbc->ansi_charset_info->csname));
+              }
+              /* We have only added the introducer, data is added below. */
+              break;
+            }
         case SQL_TIME:
         case SQL_TYPE_TIME:
-            if ( param->CType == SQL_C_TIMESTAMP ||
-                 param->CType == SQL_C_TYPE_TIMESTAMP )
+            if ( aprec->concise_type == SQL_C_TIMESTAMP ||
+                 aprec->concise_type == SQL_C_TYPE_TIMESTAMP )
             {
-                TIMESTAMP_STRUCT *time= (TIMESTAMP_STRUCT*) param->buffer;
+                TIMESTAMP_STRUCT *time= (TIMESTAMP_STRUCT*) aprec->data_ptr;
                 sprintf(buff,"'%02d:%02d:%02d'",time->hour,time->minute,time->second);
-                return add_to_buffer(net,to,buff,10);
+                to= add_to_buffer(net, to, buff, 10);
             }
             else
             {
@@ -454,8 +549,9 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
                         (int) time/10000,
                         (int) time/100%100,
                         (int) time%100);
-                return add_to_buffer(net,to,buff,10);
+                to= add_to_buffer(net, to, buff, 10);
             }
+            goto out;
         case SQL_FLOAT:
         case SQL_REAL:
         case SQL_DOUBLE:
@@ -486,7 +582,10 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
             /* Fall through */
         default:
           if (!convert)
-            return add_to_buffer(net,to,data,length);
+          {
+            to= add_to_buffer(net, to, data, length);
+            goto out;
+          }
     }
 
     to= add_to_buffer(net,to,"'",1);
@@ -495,7 +594,16 @@ char *insert_param(DBC *dbc, char *to,PARAM_BIND *param)
       return 0;
     to+= mysql_real_escape_string(&dbc->mysql, to, data, length);
     to= add_to_buffer(net, to, "'", 1);
-    return to;
+
+out:
+    if (free_data)
+      my_free(data, MYF(0));
+
+    *toptr= to;
+    return SQL_SUCCESS;
+
+memerror:
+    return set_error(stmt, MYERR_S1001, NULL, 4001);
 }
 
 
@@ -566,10 +674,9 @@ SQLRETURN SQL_API SQLExecute(SQLHSTMT hstmt)
 SQLRETURN my_SQLExecute( STMT FAR *pStmt )
 {
     char       *query, *cursor_pos;
-    uint        i;
-    uint        nIndex;
-    PARAM_BIND *param;
+    int         dae_rec;
     STMT FAR *  pStmtCursor = pStmt;
+    SQLRETURN rc;
 
     if ( !pStmt )
         return SQL_ERROR;
@@ -580,6 +687,10 @@ SQLRETURN my_SQLExecute( STMT FAR *pStmt )
         return set_error(pStmt, MYERR_S1010,
                          "No previous SQLPrepare done", 0);
 
+    if (is_set_names_statement((SQLCHAR *)pStmt->query))
+      return set_error(pStmt, MYERR_42000,
+                       "SET NAMES not allowed by driver", 0);
+
     if ( (cursor_pos= check_if_positioned_cursor_exists(pStmt, &pStmtCursor)) )
     {
       /* Save a copy of the query, because we're about to modify it. */
@@ -597,21 +708,10 @@ SQLRETURN my_SQLExecute( STMT FAR *pStmt )
       return do_my_pos_cursor(pStmt, pStmtCursor);
     }
 
-    for ( nIndex= 0 ; nIndex < pStmt->param_count ; )
-    {
-        param= dynamic_element(&pStmt->params,nIndex++,PARAM_BIND*);
-        if ( param->real_param_done == FALSE && param->used == 1 )
-        {
-            pthread_mutex_lock(&pStmt->dbc->lock);
-            mysql_free_result(pStmt->result);
-            pthread_mutex_unlock(&pStmt->dbc->lock);
-            break;
-        }
-    }
-
-    if ( pStmt->dummy_state == ST_DUMMY_EXECUTED )
-        pStmt->state= ST_PREPARED;
-    if ( pStmt->state == ST_PRE_EXECUTED )
+    /* If this statement has been executed, there are no
+     * parameters, we do not need to execute it again */
+    if (pStmt->state == ST_PRE_EXECUTED &&
+        pStmt->dummy_state != ST_DUMMY_EXECUTED)
     {
         pStmt->state= ST_EXECUTED;
         return SQL_SUCCESS;
@@ -619,80 +719,30 @@ SQLRETURN my_SQLExecute( STMT FAR *pStmt )
     my_SQLFreeStmt((SQLHSTMT)pStmt,MYSQL_RESET_BUFFERS);
     query= pStmt->query;
 
-    if ( pStmt->stmt_options.paramProcessedPtr )
-        *pStmt->stmt_options.paramProcessedPtr= 0;
+    if ( pStmt->apd->rows_processed_ptr )
+        *pStmt->apd->rows_processed_ptr= 0;
 
     if ( pStmt->param_count )
     {
-        /*
-         * If any parameters are required at execution time, cannot perform the
-         * statement. It will be done throught SQLPutData() and SQLParamData().
-         */
-        for ( i= 0; i < pStmt->param_count; i++ )
-        {
-            PARAM_BIND *param= dynamic_element(&pStmt->params,i,PARAM_BIND*);
-            if ( param->actual_len &&
-                 (*param->actual_len == (long) SQL_DATA_AT_EXEC ||
-                  *param->actual_len <= SQL_LEN_DATA_AT_EXEC_OFFSET) )
-            {
-                pStmt->current_param= i;      /* Fix by Giovanni */
-                param->value= 0;
-                param->alloced= 0;
-                return SQL_NEED_DATA;
-            }
-        }
-        query= insert_params(pStmt);     /* Checked in do_query */
+      /*
+       * If any parameters are required at execution time, cannot perform the
+       * statement. It will be done through SQLPutData() and SQLParamData().
+       */
+      if ((dae_rec= desc_find_dae_rec(pStmt->apd)) > -1)
+      {
+        pStmt->current_param= dae_rec;
+        pStmt->dae_type= DAE_NORMAL;
+        return SQL_NEED_DATA;
+      }
+
+      if (!SQL_SUCCEEDED(rc= insert_params(pStmt, &query)))
+        return rc;
     }
 
-    return do_query(pStmt, query);
-}
-
-
-/*
-  @type    : ODBC 1.0 API
-  @purpose : executes a preparable statement, using the current values of
-  the parameter marker variables if any parameters exist in the
-  statement
-*/
-
-SQLRETURN SQL_API SQLExecDirect(SQLHSTMT hstmt,
-                                SQLCHAR FAR *szSqlStr,
-                                SQLINTEGER cbSqlStr)
-{
-    int error;
-
-    if ( (error= my_SQLPrepare(hstmt,szSqlStr,cbSqlStr)) )
-        return error;
-    error= my_SQLExecute( (STMT FAR*)hstmt );
-
-    return error;
-}
-
-
-/*
-  @type    : ODBC 1.0 API
-  @purpose : returns the SQL string as modified by the driver
-*/
-
-SQLRETURN SQL_API SQLNativeSql(SQLHDBC hdbc,
-                               SQLCHAR *szSqlStrIn,
-                               SQLINTEGER cbSqlStrIn,
-                               SQLCHAR *szSqlStr,
-                               SQLINTEGER cbSqlStrMax,
-                               SQLINTEGER *pcbSqlStr)
-{
-  if (cbSqlStrIn == SQL_NTS)
-    cbSqlStrIn= strlen((char *)szSqlStrIn);
-
-  if (pcbSqlStr)
-    *pcbSqlStr= cbSqlStrIn;
-
-  (void)strncpy((char *)szSqlStr, (const char *)szSqlStrIn, cbSqlStrMax);
-
-  if (cbSqlStrIn > cbSqlStrMax)
-    return set_conn_error((DBC *)hdbc, MYERR_01004, NULL, 0);
-
-  return SQL_SUCCESS;
+    rc= do_query(pStmt, query);
+    if (pStmt->dummy_state == ST_DUMMY_PREPARED)
+        pStmt->dummy_state= ST_DUMMY_EXECUTED;
+    return rc;
 }
 
 
@@ -706,23 +756,81 @@ SQLRETURN SQL_API SQLParamData(SQLHSTMT hstmt, SQLPOINTER FAR *prbgValue)
 {
     STMT FAR *stmt= (STMT FAR*) hstmt;
     uint i;
+    SQLRETURN rc;
+    char *query;
+    DESC *apd;
+    uint param_count= stmt->param_count;
 
-    for ( i= stmt->current_param; i < stmt->param_count; i++ )
+    assert(stmt->dae_type);
+    /* get the correct APD for the dae type we're handling */
+    switch(stmt->dae_type)
     {
-        PARAM_BIND *param= dynamic_element(&stmt->params,i,PARAM_BIND*);
-        if ( param->actual_len &&
-             (*param->actual_len == (long) SQL_DATA_AT_EXEC ||
-              *param->actual_len <= SQL_LEN_DATA_AT_EXEC_OFFSET) )
+    case DAE_NORMAL:
+      apd= stmt->apd;
+      break;
+    case DAE_SETPOS_INSERT:
+    case DAE_SETPOS_UPDATE:
+      apd= stmt->setpos_apd;
+      param_count= stmt->ard->count;
+      break;
+    default:
+      return set_stmt_error(stmt, "HY010",
+                            "Invalid data at exec state", 0);
+    }
+
+    for ( i= stmt->current_param; i < param_count; i++ )
+    {
+        DESCREC *aprec= desc_get_rec(apd, i, FALSE);
+        SQLLEN *octet_length_ptr;
+
+        assert(aprec);
+        octet_length_ptr= ptr_offset_adjust(aprec->octet_length_ptr,
+                                            apd->bind_offset_ptr,
+                                            apd->bind_type,
+                                            sizeof(SQLLEN), 0);
+
+        /* get the "placeholder" pointer the application bound */
+        if (IS_DATA_AT_EXEC(octet_length_ptr))
         {
+            SQLINTEGER default_size= bind_length(aprec->concise_type,
+                                                 aprec->octet_length);
             stmt->current_param= i+1;
             if ( prbgValue )
-                *prbgValue= param->buffer;
-            param->value= 0;
-            param->alloced= 0;
+                *prbgValue= ptr_offset_adjust(aprec->data_ptr,
+                                              apd->bind_offset_ptr,
+                                              apd->bind_type,
+                                              default_size, 0);
+            aprec->par.value= NULL;
+            aprec->par.alloced= FALSE;
+            aprec->par.is_dae= 1;
             return SQL_NEED_DATA;
         }
     }
-    return do_query(stmt,insert_params(stmt));
+
+    /* all data-at-exec params are complete. continue execution */
+    switch(stmt->dae_type)
+    {
+    case DAE_NORMAL:
+      if (!SQL_SUCCEEDED(rc= insert_params(stmt, &query)))
+        break;
+      rc= do_query(stmt, query);
+      break;
+    case DAE_SETPOS_INSERT:
+      stmt->dae_type= DAE_SETPOS_DONE;
+      rc= my_SQLSetPos(hstmt, stmt->setpos_row, SQL_ADD, stmt->setpos_lock);
+      desc_free(stmt->setpos_apd);
+      stmt->setpos_apd= NULL;
+      break;
+    case DAE_SETPOS_UPDATE:
+      stmt->dae_type= DAE_SETPOS_DONE;
+      rc= my_SQLSetPos(hstmt, stmt->setpos_row, SQL_UPDATE, stmt->setpos_lock);
+      desc_free(stmt->setpos_apd);
+      stmt->setpos_apd= NULL;
+      break;
+    }
+    
+    stmt->dae_type= 0;
+    return rc;
 }
 
 
@@ -739,65 +847,112 @@ SQLRETURN SQL_API SQLPutData( SQLHSTMT      hstmt,
                               SQLLEN        cbValue )
 {
     STMT FAR *stmt= (STMT FAR*) hstmt;
-    PARAM_BIND *param;
+    DESCREC *aprec;
 
     if ( !stmt )
         return SQL_ERROR;
 
     if ( cbValue == SQL_NTS )
         cbValue= strlen(rgbValue);
-    param= dynamic_element(&stmt->params,stmt->current_param-1,PARAM_BIND*);
+    if (stmt->dae_type == DAE_NORMAL)
+      aprec= desc_get_rec(stmt->apd, stmt->current_param - 1, FALSE);
+    else
+      aprec= desc_get_rec(stmt->setpos_apd, stmt->current_param - 1, FALSE);
+    assert(aprec);
     if ( cbValue == SQL_NULL_DATA )
     {
-        if ( param->alloced )
-            my_free(param->value,MYF(0));
-        param->alloced= 0;
-        param->value= 0;
+        if ( aprec->par.alloced )
+            my_free(aprec->par.value,MYF(0));
+        aprec->par.alloced= FALSE;
+        aprec->par.value= NULL;
         return SQL_SUCCESS;
     }
-    if ( param->value )
+    if ( aprec->par.value )
     {
         /* Append to old value */
-        if ( param->alloced )
-        {
-            if ( !(param->value= my_realloc(param->value,
-                                            param->value_length + cbValue + 1,
+        assert(aprec->par.alloced);
+        if ( !(aprec->par.value= my_realloc(aprec->par.value,
+                                            aprec->par.value_length + cbValue + 1,
                                             MYF(0))) )
-                return set_error(stmt,MYERR_S1001,NULL,4001);
-        }
-        else
-        {
-            /* This should never happen */
-            char *old_pos= param->value;
-            if ( !(param->value= my_malloc(param->value_length+cbValue+1,MYF(0))) )
-                return set_error(stmt,MYERR_S1001,NULL,4001);
-            memcpy(param->value,old_pos,param->value_length);
-        }
-        memcpy(param->value+param->value_length,rgbValue,cbValue);
-        param->value_length+= cbValue;
-        param->value[param->value_length]= 0;
-        param->alloced= 1;
+          return set_error(stmt,MYERR_S1001,NULL,4001);
+
+        memcpy(aprec->par.value+aprec->par.value_length,rgbValue,cbValue);
+        aprec->par.value_length+= cbValue;
+        aprec->par.value[aprec->par.value_length]= 0;
+        aprec->par.alloced= TRUE;
     }
     else
     {
         /* New value */
-        if ( !(param->value= my_malloc(cbValue+1,MYF(0))) )
+        if ( !(aprec->par.value= my_malloc(cbValue+1,MYF(0))) )
             return set_error(stmt,MYERR_S1001,NULL,4001);
-        memcpy(param->value,rgbValue,cbValue);
-        param->value_length= cbValue;
-        param->value[param->value_length]= 0;
-        param->alloced= 1;
+        memcpy(aprec->par.value,rgbValue,cbValue);
+        aprec->par.value_length= cbValue;
+        aprec->par.value[aprec->par.value_length]= 0;
+        aprec->par.alloced= TRUE;
     }
     return SQL_SUCCESS;
 }
 
 
-/*
-  @type    : ODBC 1.0 API
-  @purpose : cancels the processing on a statement
-*/
+/**
+  Cancel the query by opening another connection and using KILL when called
+  from another thread while the query lock is being held. Otherwise, treat as
+  SQLFreeStmt(hstmt, SQL_CLOSE).
 
+  @param[in]  hstmt  Statement handle
+
+  @return Standard ODBC result code
+*/
 SQLRETURN SQL_API SQLCancel(SQLHSTMT hstmt)
 {
-    return my_SQLFreeStmt(hstmt,SQL_CLOSE);
+  DBC *dbc= ((STMT *)hstmt)->dbc;
+  MYSQL *second= NULL;
+  int error;
+
+  error= pthread_mutex_trylock(&dbc->lock);
+
+  /* If there's no query going on, just close the statement. */
+  if (error == 0)
+  {
+    pthread_mutex_unlock(&dbc->lock);
+    return my_SQLFreeStmt(hstmt, SQL_CLOSE);
+  }
+
+  /* If we got a non-BUSY error, it's just an error. */
+  if (error != EBUSY)
+    return set_stmt_error((STMT *)hstmt, "HY000",
+                          "Unable to get connection mutex status", error);
+
+  /*
+    If the mutex was locked, we need to make a new connection and KILL the
+    ongoing query.
+  */
+  second= mysql_init(second);
+
+  /** @todo need to preserve and use ssl params */
+
+  if (!mysql_real_connect(second, dbc->ds->server8, dbc->ds->uid8,
+                          dbc->ds->pwd8, NULL, dbc->ds->port,
+                          dbc->ds->socket8, 0))
+  {
+    /* We do not set the SQLSTATE here, per the ODBC spec. */
+    return SQL_ERROR;
+  }
+
+  {
+    char buff[40];
+    /* buff is always big enough because max length of %lu is 15 */
+    sprintf(buff, "KILL /*!50000 QUERY */ %lu", mysql_thread_id(&dbc->mysql));
+    if (mysql_real_query(second, buff, strlen(buff)))
+    {
+      mysql_close(second);
+      /* We do not set the SQLSTATE here, per the ODBC spec. */
+      return SQL_ERROR;
+    }
+  }
+
+  mysql_close(second);
+
+  return SQL_SUCCESS;
 }
