@@ -25,7 +25,7 @@
   @brief Transaction processing functions.
 */
 
-#include "myodbc3.h"
+#include "driver.h"
 
 /**
   Commit or roll back the transactions associated with a particular
@@ -42,7 +42,7 @@ static SQLRETURN my_transact(SQLHDBC hdbc, SQLSMALLINT CompletionType)
   const char *query;
   uint	length;
 
-  if (dbc && !(dbc->flag & FLAG_NO_TRANSACTIONS))
+  if (dbc && !dbc->ds->disable_transactions)
   {
     switch(CompletionType) {
     case SQL_COMMIT:
