@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# mysql-connector-odbc 3.51 RPM specification
+# mysql-connector-odbc 5.1 RPM specification
 #
 ##############################################################################
 
@@ -24,9 +24,9 @@ Name:       mysql-connector-odbc-commercial
 %else
 Name:       mysql-connector-odbc
 %endif
-Summary:    An ODBC 3.51 driver for MySQL - driver package
+Summary:    An ODBC 5.1 driver for MySQL - driver package
 Group:      Applications/Databases
-Version:    @VERSION@
+Version:    @NUMERIC_VERSION@
 Release:    0
 Provides:   mysqlodbcrpmpack
 %if %{com_lic}
@@ -34,17 +34,17 @@ Copyright:  Commercial
 %else
 Copyright:  GPL
 %endif
-Source0:    %{name}-%{version}.tar.gz
+Source0:    %{name}-@VERSION@.tar.gz
 URL:        http://www.mysql.com/
 Vendor:     MySQL AB
 Packager:   MySQL Production Engineering Team <build@mysql.com>
 
-BuildRoot:  %{_tmppath}/%{name}-%{version}-build
+BuildRoot:  %{_tmppath}/%{name}-@VERSION@-build
 
 %if %{no_odbc_gui}
 %else
 %package setup
-Summary:    An ODBC 3.51 driver for MySQL - setup library
+Summary:    An ODBC 5.1 driver for MySQL - setup library
 Group:      Application/Databases
 Requires:   mysqlodbcrpmpack
 %endif
@@ -61,9 +61,9 @@ mysql-connector-odbc is an ODBC (3.50) level 0 (with level 1 and level
 mysql-connector-odbc  works on Windows NT/2000/XP/2003, and most Unix
 platforms (incl. OSX and Linux).
 
-mysql-connector-odbc 3.51 is an enhanced version of MyODBC 2.50 to meet
+mysql-connector-odbc 5.1 is an enhanced version of MyODBC 2.50 to meet
 ODBC 3.5 specification. The driver is commonly referred to as
-'MySQL ODBC 3.51 Driver'.
+'MySQL ODBC 5.1 Driver'.
 
 %description setup
 The setup library for the MySQL ODBC package, handles the optional GUI
@@ -76,7 +76,7 @@ dialog for configuring the driver.
 ##############################################################################
 
 %prep
-%setup
+%setup -n %{name}-@VERSION@
 
 %define ODBC_DM_PATH_ARG @ODBC_DM_PATH_ARG@
 %define MYSQL_PATH_ARG   @MYSQL_PATH_ARG@
@@ -132,11 +132,11 @@ rm -v $RPM_BUILD_ROOT%{_datadir}/mysql-connector-odbc/{ChangeLog,README,README.d
 # ----------------------------------------------------------------------
 
 %post 
-myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=%{_libdir}/libmyodbc3.so"
+myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodbc5.so"
 
 %post setup
-myodbc3i -r -d -n"MySQL ODBC 3.51 Driver"
-myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=%{_libdir}/libmyodbc3.so;SETUP=%{_libdir}/libmyodbc3S.so"
+myodbc-installer -r -d -n "MySQL ODBC 5.1 Driver"
+myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodbc5.so;SETUP=%{_libdir}/libmyodbc3S.so"
 
 # ----------------------------------------------------------------------
 # DEREGISTER DRIVER 
@@ -144,12 +144,12 @@ myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=%{_libdir}/libmyodbc3.so;SETUP=%
 
 # Removing the driver package, we simply orphan any related DSNs
 %preun
-myodbc3i -r -d -n"MySQL ODBC 3.51 Driver"
+myodbc-installer -r -d -n "MySQL ODBC 5.1 Driver"
 
 # Removing the setup RPM, downgrade the registration
 %preun setup
-myodbc3i -r -d -n"MySQL ODBC 3.51 Driver"
-myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=%{_libdir}/libmyodbc3.so"
+myodbc-installer -r -d -n "MySQL ODBC 5.1 Driver"
+myodbc-installer -a -d -n "MySQL ODBC 5.1 Driver" -t "DRIVER=%{_libdir}/libmyodbc5.so"
 
 ##############################################################################
 #
@@ -159,14 +159,12 @@ myodbc3i -a -d -t"MySQL ODBC 3.51 Driver;DRIVER=%{_libdir}/libmyodbc3.so"
 
 %files 
 %defattr(-,root,root)
-%{_bindir}/myodbc3i
-%{_bindir}/myodbc3m
-%{_libdir}/libmyodbc3.*
-%{_libdir}/libmyodbc3-*
-%{_libdir}/libmyodbc3_*
+%{_bindir}/myodbc-installer
+%{_libdir}/libmyodbc5.*
+%{_libdir}/libmyodbc5-*
 %doc ChangeLog README README.debug INSTALL
 %if %{com_lic}
-%doc LICENSE.commercial
+%doc LICENSE.mysql
 %else
 %doc LICENSE.gpl
 %doc LICENSE.exceptions
