@@ -24,26 +24,20 @@
 
 DECLARE_TEST(my_columns_null)
 {
-    SQLRETURN   rc;
+  /* initialize data */
+  ok_sql(hstmt,"drop table if exists my_column_null");
 
-    /* initialize data */
-    SQLExecDirect(hstmt,"drop table my_column_null",SQL_NTS);
+  ok_sql(hstmt, "create table my_column_null(id int not null, name varchar(30))");
 
-    rc = SQLExecDirect(hstmt,"create table my_column_null(id int not null, name varchar(30))",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, SQL_NTS, NULL, SQL_NTS,
+                            (SQLCHAR *)"my_column_null", SQL_NTS,
+                            NULL, SQL_NTS));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  is(2 == my_print_non_format_result(hstmt));
 
-    rc = SQLColumns(hstmt,NULL,SQL_NTS,NULL,SQL_NTS,"my_column_null",SQL_NTS,NULL,SQL_NTS);
-    mystmt(hstmt,rc);
-
-    myassert(2 == my_print_non_format_result(hstmt));
-
-    SQLFreeStmt(hstmt, SQL_CLOSE);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS my_column_null");
 
@@ -53,30 +47,17 @@ DECLARE_TEST(my_columns_null)
 
 DECLARE_TEST(my_drop_table)
 {
-    SQLRETURN   rc;
+  ok_sql(hstmt, "drop table if exists my_drop_table");
+  ok_sql(hstmt, "create table my_drop_table(id int not null)");
 
-    /* initialize data */
-    SQLExecDirect(hstmt,"drop table my_drop_table",SQL_NTS);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-    rc = SQLExecDirect(hstmt,"create table my_drop_table(id int not null)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"my_drop_table", SQL_NTS, NULL, 0));
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  is(1 == my_print_non_format_result(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
-
-    rc = SQLColumns(hstmt,NULL,0,NULL,0,"my_drop_table",SQL_NTS,NULL,0);
-    mystmt(hstmt,rc);
-
-    myassert(1 == my_print_non_format_result(hstmt));
-
-    rc = SQLExecDirect(hstmt,"drop table my_drop_table",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "drop table my_drop_table");
 
   return OK;
 }
@@ -92,87 +73,84 @@ DECLARE_TEST(my_table_dbs)
     SQLINTEGER nrows;
     SQLLEN lenOrNull;
 
-    SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test1",   SQL_NTS);
-    SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test2",   SQL_NTS);
-    SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test3",   SQL_NTS);
-    SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test4",   SQL_NTS);
+  ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test1");
+  ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test2");
+  ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test3");
+  ok_sql(hstmt, "DROP DATABASE IF EXISTS my_all_db_test4");
 
-    rc = SQLTables(hstmt,"%",1,NULL,0,NULL,0,NULL,0);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLTables(hstmt,(SQLCHAR *)"%",1,"",0,"",0,NULL,0));
 
     nrows = my_print_non_format_result(hstmt);
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"SQL_ALL_CATALOGS",SQL_NTS,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt,(SQLCHAR *)SQL_ALL_CATALOGS,SQL_NTS,"",0,"",0,
+                   NULL,0);
     mystmt(hstmt,rc);
 
     is(nrows == my_print_non_format_result(hstmt));
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"test",4,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt,(SQLCHAR *)"test",4,NULL,0,NULL,0,NULL,0);
     mystmt(hstmt,rc);
 
     my_print_non_format_result(hstmt);
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"mysql",5,NULL,0,NULL,0,NULL,0);
+    /* test fails on Win2003 x86 w/DM if len=5, SQL_NTS is used instead */
+    rc = SQLTables(hstmt,(SQLCHAR *)"mysql",SQL_NTS,NULL,0,NULL,0,NULL,0);
     mystmt(hstmt,rc);
 
     is(my_print_non_format_result(hstmt) != 0);
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"%",1,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt,(SQLCHAR *)"%",1,"",0,"",0,NULL,0);
     mystmt(hstmt,rc);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,1,SQL_C_CHAR,(SQLCHAR *)&database,100,NULL);
+    rc = SQLGetData(hstmt,1,SQL_C_CHAR,database,100,NULL);
     mystmt(hstmt,rc);
-    printMessage("\n catalog: %s", database);
+    printMessage("catalog: %s", database);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,2,SQL_C_CHAR,(SQLCHAR *)&database,100,&lenOrNull);
+    rc = SQLGetData(hstmt,2,SQL_C_CHAR,database,100,&lenOrNull);
     mystmt(hstmt,rc);
-    printMessage("\n schema: %s", database); 
+    printMessage("schema: %s", database);
     myassert(lenOrNull == SQL_NULL_DATA);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,3,SQL_C_CHAR,(SQLCHAR *)&database,100,&lenOrNull);
+    rc = SQLGetData(hstmt,3,SQL_C_CHAR,database,100,&lenOrNull);
     mystmt(hstmt,rc);
-    printMessage("\n table: %s", database); 
+    printMessage("table: %s", database);
     myassert(lenOrNull == SQL_NULL_DATA);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,4,SQL_C_CHAR,(SQLCHAR *)&database,100,&lenOrNull);
+    rc = SQLGetData(hstmt,4,SQL_C_CHAR,database,100,&lenOrNull);
     mystmt(hstmt,rc);
-    printMessage("\n type: %s", database); 
+    printMessage("type: %s", database);
     myassert(lenOrNull == SQL_NULL_DATA);
 
     memset(database,0,100);
-    rc = SQLGetData(hstmt,5,SQL_C_CHAR, (SQLCHAR*)&database,100,&lenOrNull);
+    rc = SQLGetData(hstmt,5,SQL_C_CHAR, database,100,&lenOrNull);
     mystmt(hstmt,rc);
-    printMessage("\n database remark: %s", database);
+    printMessage("database remark: %s", database);
     myassert(lenOrNull == SQL_NULL_DATA);
 
     SQLFreeStmt(hstmt,SQL_UNBIND);
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
-    rc = SQLExecDirect(hstmt, "CREATE DATABASE my_all_db_test1",    SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "CREATE DATABASE my_all_db_test2",    SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "CREATE DATABASE my_all_db_test3",    SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "CREATE DATABASE my_all_db_test4",    SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "CREATE DATABASE my_all_db_test1");
+    ok_sql(hstmt, "CREATE DATABASE my_all_db_test2");
+    ok_sql(hstmt, "CREATE DATABASE my_all_db_test3");
+    ok_sql(hstmt, "CREATE DATABASE my_all_db_test4");
 
-    rc = SQLTables(hstmt,"%",1,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt, (SQLCHAR *)"%", 1, "", 0, "", 0, "", 0);
     mystmt(hstmt,rc);
 
     nrows += 4;
@@ -180,43 +158,43 @@ DECLARE_TEST(my_table_dbs)
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"SQL_ALL_CATALOGS",SQL_NTS,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt,(SQLCHAR *)SQL_ALL_CATALOGS, SQL_NTS,
+                   "", 0, "", 0, "", 0);
     mystmt(hstmt,rc);
 
     is(nrows == my_print_non_format_result(hstmt));
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"my_all_db_test",SQL_NTS,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt, (SQLCHAR *)"my_all_db_test", SQL_NTS,
+                   "", 0, "", 0, "", 0);
     mystmt(hstmt,rc);
 
     is(0 == my_print_non_format_result(hstmt));
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLTables(hstmt,"my_all_db_test%",SQL_NTS,NULL,0,NULL,0,NULL,0);
+    rc = SQLTables(hstmt, (SQLCHAR *)"my_all_db_test%", SQL_NTS,
+                   "", 0, "", 0, NULL, 0);
     mystmt(hstmt,rc);
 
-    is(0 == my_print_non_format_result(hstmt));
+    is(4 == my_print_non_format_result(hstmt));
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
     /* unknown table should be empty */
-    rc = SQLTables(hstmt,"my_all_db_test%",SQL_NTS,NULL,0,"xyz",SQL_NTS,NULL,0);
+    rc = SQLTables(hstmt, (SQLCHAR *)"my_all_db_test%", SQL_NTS,
+                   NULL, 0, (SQLCHAR *)"xyz", SQL_NTS, NULL, 0);
     mystmt(hstmt,rc);
 
     is(0 == my_print_non_format_result(hstmt));
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test1",  SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test2",  SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test3",  SQL_NTS);
-    mystmt(hstmt,rc);
-    rc = SQLExecDirect(hstmt, "DROP DATABASE my_all_db_test4",  SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "DROP DATABASE my_all_db_test1");
+  ok_sql(hstmt, "DROP DATABASE my_all_db_test2");
+  ok_sql(hstmt, "DROP DATABASE my_all_db_test3");
+  ok_sql(hstmt, "DROP DATABASE my_all_db_test4");
 
   return OK;
 }
@@ -233,6 +211,7 @@ DECLARE_TEST(my_colpriv)
   ok_sql(hstmt, "CREATE TABLE test_colprev3(a INT,b INT,c INT, d INT)");
 
   (void)SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER my_colpriv", SQL_NTS);
+  ok_sql(hstmt, "CREATE USER my_colpriv");
 
   ok_sql(hstmt, "GRANT SELECT(a,b),INSERT(d),UPDATE(c) ON test_colprev1 TO my_colpriv");
   ok_sql(hstmt, "GRANT SELECT(c,a),UPDATE(a,b) ON test_colprev3 TO my_colpriv");
@@ -243,7 +222,8 @@ DECLARE_TEST(my_colpriv)
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
                                      NULL, SQL_NTS, NULL, SQL_NTS,
-                                     "test_colprev1", SQL_NTS, NULL, SQL_NTS));
+                                     (SQLCHAR *)"test_colprev1", SQL_NTS,
+                                     (SQLCHAR *)"%", SQL_NTS));
 
   is(4 == my_print_non_format_result(hstmt));
 
@@ -251,7 +231,8 @@ DECLARE_TEST(my_colpriv)
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
                                      NULL, SQL_NTS, NULL, SQL_NTS,
-                                     "test_colprev1", SQL_NTS, "a", SQL_NTS));
+                                     (SQLCHAR *)"test_colprev1", SQL_NTS,
+                                     (SQLCHAR *)"a", SQL_NTS));
 
   is(1 == my_print_non_format_result(hstmt));
 
@@ -259,14 +240,16 @@ DECLARE_TEST(my_colpriv)
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
                                      NULL, SQL_NTS, NULL, SQL_NTS,
-                                     "test_colprev2", SQL_NTS, NULL, SQL_NTS));
+                                     (SQLCHAR *)"test_colprev2", SQL_NTS,
+                                     (SQLCHAR *)"%", SQL_NTS));
   is(0 == my_print_non_format_result(hstmt));
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
                                      NULL, SQL_NTS, NULL, SQL_NTS,
-                                     "test_colprev3", SQL_NTS, NULL, SQL_NTS));
+                                     (SQLCHAR *)"test_colprev3", SQL_NTS,
+                                     (SQLCHAR *)"%", SQL_NTS));
 
   is(4 == my_print_non_format_result(hstmt));
 
@@ -274,23 +257,17 @@ DECLARE_TEST(my_colpriv)
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
                                      NULL, SQL_NTS, NULL, SQL_NTS,
-                                     "test_%", SQL_NTS, NULL, SQL_NTS));
+                                     (SQLCHAR *)"test_%", SQL_NTS,
+                                     (SQLCHAR *)"%", SQL_NTS));
 
-  my_print_non_format_result(hstmt);
+  is(0 == my_print_non_format_result(hstmt));
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
-                                     NULL, SQL_NTS, NULL, SQL_NTS,
-                                     "test_colprev%", SQL_NTS, NULL, SQL_NTS));
-
-  is(8 == my_print_non_format_result(hstmt));
-
-  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-
-  ok_stmt(hstmt, SQLColumnPrivileges(hstmt,
-                                     "mysql", SQL_NTS, NULL, SQL_NTS,
-                                     "columns_priv", SQL_NTS, NULL, SQL_NTS));
+                                     (SQLCHAR *)"mysql", SQL_NTS, NULL, SQL_NTS,
+                                     (SQLCHAR *)"columns_priv", SQL_NTS,
+                                     NULL, SQL_NTS));
 
   my_print_non_format_result(hstmt);
 
@@ -325,13 +302,14 @@ DECLARE_TEST(t_sqlprocedures)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   /* Try without specifying a catalog. */
-  ok_stmt(hstmt, SQLProcedures(hstmt, NULL, 0, NULL, 0, "t_sqlproc%", SQL_NTS));
+  ok_stmt(hstmt, SQLProcedures(hstmt, NULL, 0, NULL, 0,
+                               (SQLCHAR *)"t_sqlproc%", SQL_NTS));
 
   is_num(my_print_non_format_result(hstmt), 2);
 
   /* And try with specifying a catalog.  */
-  ok_stmt(hstmt, SQLProcedures(hstmt, "test", SQL_NTS, NULL, 0,
-                               "t_sqlproc%", SQL_NTS));
+  ok_stmt(hstmt, SQLProcedures(hstmt, (SQLCHAR *)"test", SQL_NTS, NULL, 0,
+                               (SQLCHAR *)"t_sqlproc%", SQL_NTS));
 
   is_num(my_print_non_format_result(hstmt), 2);
 
@@ -360,25 +338,25 @@ DECLARE_TEST(t_catalog)
         9,11,10,11,9,9,11,13,14,14,8,7,10,13,16,17,16,11
     };
 
-    SQLExecDirect(hstmt,"drop table t_catalog",SQL_NTS);
+    ok_sql(hstmt, "drop table if exists t_catalog");
 
-    rc = SQLExecDirect(hstmt,"create table t_catalog(a tinyint, b char(4))",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt,"create table t_catalog(a tinyint, b char(4))");
 
-    rc = SQLColumns(hstmt,NULL,0,NULL,0,"t_catalog",9,NULL,0);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                              (SQLCHAR *)"t_catalog", 9, NULL, 0));
 
     rc = SQLNumResultCols(hstmt, &ncols);
     mystmt(hstmt,rc);
 
-    printMessage("\n total columns: %d", ncols);
+    printMessage("total columns: %d", ncols);
     myassert(ncols == 18);
     myassert(myresult(hstmt) == 2);
 
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
-    rc = SQLColumns(hstmt,NULL,0,NULL,0,"t_catalog",9,NULL,0);
+    rc = SQLColumns(hstmt, NULL, 0, NULL, 0,
+                    (SQLCHAR *)"t_catalog", 9, NULL, 0);
     mystmt(hstmt,rc);
 
     rc = SQLNumResultCols(hstmt,&ncols);
@@ -389,8 +367,9 @@ DECLARE_TEST(t_catalog)
         rc = SQLDescribeCol(hstmt, i, name, MYSQL_NAME_LEN+1, &len, NULL, NULL, NULL, NULL);
         mystmt(hstmt,rc);
 
-        printMessage("\n column %d: %s (%d)", i, name, len);
-        myassert(strcmp(name,colnames[i-1]) == 0 && len == collengths[i-1]);
+        printMessage("column %d: %s (%d)", i, name, len);
+        is_num(len, collengths[i - 1]);
+        is_str(name, colnames[i - 1], len);
     }
     SQLFreeStmt(hstmt,SQL_CLOSE);
 
@@ -431,14 +410,9 @@ DECLARE_TEST(tmysql_specialcols)
     rc = SQLFreeStmt(hstmt,SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLSpecialColumns(hstmt,
-                          SQL_BEST_ROWID,
-                          NULL,0,
-                          NULL,0,
-                          "tmysql_specialcols",SQL_NTS,
-                          SQL_SCOPE_SESSION,
-                          SQL_NULLABLE);
-    mystmt(hstmt,rc);
+    ok_stmt(hstmt, SQLSpecialColumns(hstmt, SQL_BEST_ROWID, NULL,0, NULL,0,
+                                     (SQLCHAR *)"tmysql_specialcols",SQL_NTS,
+                                     SQL_SCOPE_SESSION, SQL_NULLABLE));
 
     myresult(hstmt);
 
@@ -570,60 +544,53 @@ typedef struct t_table_bug
 
 t_describe_col t_tables_bug_data[5] =
 {
-  {"TABLE_CAT",   9, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
-  {"TABLE_SCHEM",11, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
-  {"TABLE_NAME", 10, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
-  {"TABLE_TYPE", 10, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
-  {"REMARKS",     7, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_CAT",   9, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_SCHEM",11, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_NAME", 10, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_TYPE", 10, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"REMARKS",     7, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
 };
 
 
 DECLARE_TEST(t_tables_bug)
 {
-  SQLRETURN   rc;
   SQLSMALLINT i, ColumnCount, pcbColName, pfSqlType, pibScale, pfNullable;
   SQLULEN     pcbColDef;
   SQLCHAR     szColName[MAX_NAME_LEN];
-  int is51= mysql_min_version(hdbc, "5.1", 3);
 
-   SQLFreeStmt(hstmt, SQL_CLOSE);
+  ok_stmt(hstmt,  SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"'TABLE'", SQL_NTS));
 
-   rc = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,"'TABLE'",SQL_NTS);
-   mystmt(hstmt,rc);
-
-   rc = SQLNumResultCols(hstmt,&ColumnCount);
-   mystmt(hstmt,rc);
-
-   fprintf(stdout, "total columns in SQLTables: %d\n", ColumnCount);
-   myassert(ColumnCount == 5);
+   ok_stmt(hstmt, SQLNumResultCols(hstmt, &ColumnCount));
+   is_num(ColumnCount, 5);
 
    for (i= 1; i <= ColumnCount; i++)
    {
-     rc = SQLDescribeCol(hstmt, (SQLUSMALLINT)i,
-                         szColName,MAX_NAME_LEN,&pcbColName,
-                         &pfSqlType,&pcbColDef,&pibScale,&pfNullable);
-     mystmt(hstmt,rc);
+     ok_stmt(hstmt, SQLDescribeCol(hstmt, (SQLUSMALLINT)i, szColName,
+                                   MAX_NAME_LEN, &pcbColName, &pfSqlType,
+                                   &pcbColDef, &pibScale, &pfNullable));
 
-     fprintf(stdout, "Column Number'%d':\n", i);
-     fprintf(stdout, "\t Column Name    : %s\n", szColName);
-     fprintf(stdout, "\t NameLengh      : %d\n", pcbColName);
-     fprintf(stdout, "\t DataType       : %d\n", pfSqlType);
-     fprintf(stdout, "\t ColumnSize     : %d\n", pcbColDef);
-     fprintf(stdout, "\t DecimalDigits  : %d\n", pibScale);
-     fprintf(stdout, "\t Nullable       : %d\n", pfNullable);
+     fprintf(stdout, "# Column '%d':\n", i);
+     fprintf(stdout, "#  Column Name   : %s\n", szColName);
+     fprintf(stdout, "#  NameLengh     : %d\n", pcbColName);
+     fprintf(stdout, "#  DataType      : %d\n", pfSqlType);
+     fprintf(stdout, "#  ColumnSize    : %d\n", pcbColDef);
+     fprintf(stdout, "#  DecimalDigits : %d\n", pibScale);
+     fprintf(stdout, "#  Nullable      : %d\n", pfNullable);
 
-     myassert(strcmp(t_tables_bug_data[i-1].szColName,szColName) == 0);
-     myassert(t_tables_bug_data[i-1].pcbColName == pcbColName);
-     myassert(t_tables_bug_data[i-1].pfSqlType == pfSqlType);
+     is_str(t_tables_bug_data[i-1].szColName, szColName, pcbColName);
+     is_num(t_tables_bug_data[i-1].pcbColName, pcbColName);
+     is_num(t_tables_bug_data[i-1].pfSqlType, pfSqlType);
      /* This depends on NAME_LEN in mysql_com.h */
 #if UNRELIABLE_TEST
      is(t_tables_bug_data[i-1].pcbColDef == pcbColDef ||
         t_tables_bug_data[i-1].pcbColDef == pcbColDef / 3);
 #endif
-     myassert(t_tables_bug_data[i-1].pibScale == pibScale);
-     myassert(t_tables_bug_data[i-1].pfNullable == pfNullable);
+     is_num(t_tables_bug_data[i-1].pibScale, pibScale);
+     is_num(t_tables_bug_data[i-1].pfNullable, pfNullable);
    }
-   SQLFreeStmt(hstmt,SQL_CLOSE);
+
+   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   return OK;
 }
@@ -633,50 +600,50 @@ DECLARE_TEST(t_current_catalog)
 {
   SQLCHAR     cur_db[255], db[255];
   SQLRETURN   rc;
-  SQLUINTEGER len;
+  SQLINTEGER len;
 
     rc = SQLFreeStmt(hstmt,SQL_CLOSE);
     mystmt(hstmt,rc);
 
-    rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)db, 255, &len);
+    rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, sizeof(db), &len);
     mycon(hdbc,rc);
     fprintf(stdout,"current_catalog: %s (%ld)\n", db, len);
-    myassert(strcmp(db, "test") == 0 || strlen("test") == len);
+    is_num(len, 4);
+    is_str(db, "test", 5);
 
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)db, SQL_NTS);
+    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, SQL_NTS);
     mycon(hdbc,rc);
 
-    SQLExecDirect(hstmt, "DROP DATABASE IF EXISTS test_odbc_current", SQL_NTS);
+    ok_sql(hstmt, "DROP DATABASE IF EXISTS test_odbc_current");
 
-    strcpy(cur_db, "test_odbc_current");
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)cur_db, SQL_NTS);
+    strcpy((char *)cur_db, "test_odbc_current");
+    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
     mycon_r(hdbc,rc);
 
-    rc = SQLExecDirect(hstmt, "CREATE DATABASE test_odbc_current", SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "CREATE DATABASE test_odbc_current");
 
-    strcpy(cur_db, "test_odbc_current");
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)cur_db, SQL_NTS);
+    strcpy((char *)cur_db, "test_odbc_current");
+    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
     mycon(hdbc,rc);
 
-    rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)db, 255, &len);
+    rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, 255, &len);
     mycon(hdbc,rc);
     fprintf(stdout,"current_catalog: %s (%ld)\n", db, len);
-    myassert(strcmp(cur_db, db) == 0 || strlen(cur_db) == len);
+    is_num(len, 17);
+    is_str(db, cur_db, 18);
 
-    strcpy(cur_db, "test_odbc_current_12455");
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)cur_db, SQL_NTS);
+    strcpy((char *)cur_db, "test_odbc_current_12455");
+    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
     mycon_r(hdbc,rc);
 
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (char *)cur_db, len);
+    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, len);
     mycon(hdbc,rc);
 
     /* reset for further tests */
     rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (SQLCHAR *)"test", SQL_NTS);
     mycon(hdbc,rc);
 
-    rc = SQLExecDirect(hstmt, "DROP DATABASE test_odbc_current", SQL_NTS);
-    mycon(hstmt,rc);
+  ok_sql(hstmt, "DROP DATABASE test_odbc_current");
 
   return OK;
 }
@@ -714,16 +681,17 @@ DECLARE_TEST(tmysql_showkeys)
 DECLARE_TEST(t_sqltables)
 {
     SQLRETURN r;
+    SQLINTEGER rows;
 
-    r  = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,NULL,0);
-    mystmt(hstmt,r);
+    ok_stmt(hstmt, SQLTables(hstmt,NULL,0,NULL,0,NULL,0,NULL,0));
 
     myresult(hstmt);
 
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
 
-    r  = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,"'system table'",SQL_NTS);
+    r  = SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
+                   (SQLCHAR *)"'system table'", SQL_NTS);
     mystmt(hstmt,r);
 
     is_num(myresult(hstmt), 0);
@@ -731,7 +699,8 @@ DECLARE_TEST(t_sqltables)
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
 
-    r  = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,"TABLE",SQL_NTS);
+    r  = SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
+                   (SQLCHAR *)"TABLE", SQL_NTS);
     mystmt(hstmt,r);
 
     myresult(hstmt);
@@ -739,7 +708,9 @@ DECLARE_TEST(t_sqltables)
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
 
-    r  = SQLTables(hstmt,"TEST",SQL_NTS,"TEST",SQL_NTS,NULL,0,"TABLE",SQL_NTS);
+    r  = SQLTables(hstmt, (SQLCHAR *)"TEST", SQL_NTS,
+                   (SQLCHAR *)"TEST", SQL_NTS, NULL, 0,
+                   (SQLCHAR *)"TABLE", SQL_NTS);
     mystmt(hstmt,r);
 
     myresult(hstmt);
@@ -747,7 +718,7 @@ DECLARE_TEST(t_sqltables)
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
 
-    r = SQLTables(hstmt,"%",SQL_NTS,NULL,0,NULL,0,NULL,0);
+    r = SQLTables(hstmt, (SQLCHAR *)"%", SQL_NTS, NULL, 0, NULL, 0, NULL, 0);
     mystmt(hstmt,r);
 
     myresult(hstmt);
@@ -755,7 +726,7 @@ DECLARE_TEST(t_sqltables)
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
 
-    r = SQLTables(hstmt,NULL,0,"%",SQL_NTS,NULL,0,NULL,0);
+    r = SQLTables(hstmt, NULL, 0, (SQLCHAR *)"%", SQL_NTS, NULL, 0, NULL, 0);
     mystmt(hstmt,r);
 
     myresult(hstmt);
@@ -763,10 +734,11 @@ DECLARE_TEST(t_sqltables)
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
 
-    r = SQLTables(hstmt,NULL,0,NULL,0,NULL,0,"%",SQL_NTS);
+    r = SQLTables(hstmt, "", 0, "", 0, "", 0, (SQLCHAR *)"%", SQL_NTS);
     mystmt(hstmt,r);
 
-    is_num(myresult(hstmt), 3);
+    rows= myresult(hstmt);
+    is_num(rows, 3);
 
     r = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,r);
@@ -779,16 +751,13 @@ DECLARE_TEST(t_sqltables)
  Bug #4518: SQLForeignKeys returns too many foreign key
  Bug #27723: SQLForeignKeys does not escape _ and % in the table name arguments
 
- The original test case was extended to have a table that would inadvertantly
+ The original test case was extended to have a table that would inadvertently
  get included because of the poor escaping.
 */
 DECLARE_TEST(t_bug4518)
 {
   SQLCHAR buff[255];
-
-  /** @todo re-enable this test when I_S based SQLForeignKeys is done. */
-  if (mysql_min_version(hdbc, "5.1", 3))
-    skip("can't test foreign keys with 5.1 or later yet");
+  SQLLEN len;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug4518_c, t_bug4518_c2, t_bug4518ac, "
                 "                     t_bug4518_p");
@@ -803,13 +772,15 @@ DECLARE_TEST(t_bug4518)
                 "                           FOREIGN KEY (parent_id)"
                 "                            REFERENCES"
                 "                              t_bug4518_p(id)"
-                "                            ON DELETE SET NULL)"
+                "                          )"
                 " ENGINE=InnoDB");
   ok_sql(hstmt, "CREATE TABLE t_bug4518ac (id INT, parent_id INT,"
                 "                          FOREIGN KEY (parent_id)"
                 "                           REFERENCES"
                 "                             t_bug4518_p(id)"
-                "                           ON DELETE SET NULL)"
+                "                            ON DELETE CASCADE"
+                "                            ON UPDATE NO ACTION"
+                "                          )"
                 " ENGINE=InnoDB");
 
   ok_stmt(hstmt, SQLForeignKeys(hstmt, NULL, 0, NULL, 0, NULL, 0, NULL, 0,
@@ -820,10 +791,42 @@ DECLARE_TEST(t_bug4518)
   is_str(my_fetch_str(hstmt, buff, 4), "id", 2);
   is_str(my_fetch_str(hstmt, buff, 7), "t_bug4518_c", 11);
   is_str(my_fetch_str(hstmt, buff, 8), "parent_id", 9);
+  if (mysql_min_version(hdbc, "5.1", 3))
+  {
+    is_num(my_fetch_int(hstmt, 10), SQL_NO_ACTION);
+    is_num(my_fetch_int(hstmt, 11), SQL_SET_NULL);
+  }
+  else
+  {
+    is_num(my_fetch_int(hstmt, 10), SQL_RESTRICT);
+    is_num(my_fetch_int(hstmt, 11), SQL_RESTRICT);
+  }
+
+  /* For Bug #19923: Test that schema columns are NULL. */
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, buff, sizeof(buff), &len));
+  is_num(len, SQL_NULL_DATA);
+  ok_stmt(hstmt, SQLGetData(hstmt, 6, SQL_C_CHAR, buff, sizeof(buff), &len));
+  is_num(len, SQL_NULL_DATA);
 
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
-  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLForeignKeys(hstmt, NULL, 0, NULL, 0, NULL, 0, NULL, 0,
+                                NULL, 0, (SQLCHAR *)"t_bug4518ac", SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug4518_p", 11);
+  is_str(my_fetch_str(hstmt, buff, 4), "id", 2);
+  is_str(my_fetch_str(hstmt, buff, 7), "t_bug4518ac", 11);
+  is_str(my_fetch_str(hstmt, buff, 8), "parent_id", 9);
+  if (mysql_min_version(hdbc, "5.1", 3))
+  {
+    is_num(my_fetch_int(hstmt, 10), SQL_NO_ACTION);
+    is_num(my_fetch_int(hstmt, 11), SQL_CASCADE);
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_sql(hstmt,
          "DROP TABLE t_bug4518_c, t_bug4518_c2, t_bug4518ac, t_bug4518_p");
@@ -857,6 +860,18 @@ DECLARE_TEST(empty_set)
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
+  /* SQLTables(): empty catalog with existing table */
+  ok_sql(hstmt, "drop table if exists t_sqltables_empty");
+  ok_sql(hstmt, "create table t_sqltables_empty (x int)");
+  ok_stmt(hstmt, SQLTables(hstmt, "", SQL_NTS, NULL, 0,
+			   (SQLCHAR *) "t_sqltables_empty", SQL_NTS,
+			   NULL, SQL_NTS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &columns));
+  is_num(columns, 5);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists t_sqltables_empty");
+
   return OK;
 }
 
@@ -938,7 +953,7 @@ DECLARE_TEST(bug15713)
   ok_sql(hstmt, "CREATE TABLE t_bug15713 (a INT)");
 
   /* The connection strings must not include DATABASE. */
-  sprintf((char *)conn, "DSN=%s;UID=%s;PASSWORD=%s", mydsn, myuid, mypwd);
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s", mydsn, myuid, mypwd);
   if (mysock != NULL)
   {
     strcat((char *)conn, ";SOCKET=");
@@ -1011,16 +1026,21 @@ DECLARE_TEST(bug8860)
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
-  /* Specifying "" as the table name gets us nothing. */
-  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, (SQLCHAR *)"", SQL_NTS,
-                            NULL, 0));
+  if (!using_dm(hdbc))
+  {
+    /* Specifying "" as the table name gets us nothing. */
+    /* But iODBC, for one, will convert our "" into a NULL. */
+    ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, (SQLCHAR *)"", SQL_NTS,
+                              NULL, 0));
 
-  is_num(myrowcount(hstmt), 0);
+    is(myrowcount(hstmt) == 0);
+  }
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   /* Get the info from just one table.  */
-  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, "t_bug8860", SQL_NTS,
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"t_bug8860", SQL_NTS,
                             NULL, 0));
 
   ok_stmt(hstmt, SQLFetch(hstmt));
@@ -1033,7 +1053,8 @@ DECLARE_TEST(bug8860)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   /* Get the info from just one table with a funny name.  */
-  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0, "t_bug8860_a'b", SQL_NTS,
+  ok_stmt(hstmt, SQLColumns(hstmt, NULL, 0, NULL, 0,
+                            (SQLCHAR *)"t_bug8860_a'b", SQL_NTS,
                             NULL, 0));
 
   ok_stmt(hstmt, SQLFetch(hstmt));
@@ -1061,8 +1082,8 @@ DECLARE_TEST(t_bug26934)
 
   ok_sql(hstmt1, "SET @@wait_timeout = 1");
   sleep(2);
-  expect_stmt(hstmt1, SQLTables(hstmt1, "%", 1, NULL, SQL_NTS, NULL, SQL_NTS,
-                                NULL, SQL_NTS), SQL_ERROR);
+  expect_stmt(hstmt1, SQLTables(hstmt1, (SQLCHAR *)"%", 1, NULL, SQL_NTS,
+                                NULL, SQL_NTS, NULL, SQL_NTS), SQL_ERROR);
   if (check_sqlstate(hstmt1, "08S01") != OK)
     return FAIL;
 
@@ -1124,8 +1145,8 @@ DECLARE_TEST(t_bug14407)
   */
   ok_sql(hstmt, "SELECT * FROM t_bug14407");
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, NULL, 0, NULL, NULL, NULL, NULL,
-                                &nullable));
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, col, sizeof(col), NULL, NULL, NULL,
+                                NULL, &nullable));
   is_num(nullable, SQL_NULLABLE);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -1135,6 +1156,51 @@ DECLARE_TEST(t_bug14407)
 }
 
 
+/**
+ Bug #19923: MyODBC Foreign key retrieval broken in multiple ways
+ Two issues to test: schema columns should be NULL, not just an empty string,
+ and more than one constraint should be returned.
+*/
+DECLARE_TEST(t_bug19923)
+{
+  SQLCHAR buff[255];
+  SQLLEN len;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug19923c, t_bug19923b, t_bug19923a");
+  ok_sql(hstmt, "CREATE TABLE t_bug19923a (a INT PRIMARY KEY) ENGINE=InnoDB");
+  ok_sql(hstmt, "CREATE TABLE t_bug19923b (b INT PRIMARY KEY) ENGINE=InnoDB");
+  ok_sql(hstmt, "CREATE TABLE t_bug19923c (a INT, b INT, UNIQUE(a), UNIQUE(b), CONSTRAINT `first_constraint` FOREIGN KEY (`b`) REFERENCES `t_bug19923b` (`b`), CONSTRAINT `second_constraint` FOREIGN KEY (`a`) REFERENCES `t_bug19923a` (`a`)) ENGINE=InnoDB");
+
+  ok_stmt(hstmt, SQLForeignKeys(hstmt, NULL, 0, NULL, 0, NULL, 0, NULL, 0,
+                                NULL, 0, (SQLCHAR *)"t_bug19923c", SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug19923a", 11);
+  is_str(my_fetch_str(hstmt, buff, 4), "a", 1);
+  is_str(my_fetch_str(hstmt, buff, 7), "t_bug19923c", 11);
+  is_str(my_fetch_str(hstmt, buff, 8), "a", 1);
+  is_str(my_fetch_str(hstmt, buff, 12), "second_constraint", 17);
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buff, 3), "t_bug19923b", 11);
+  is_str(my_fetch_str(hstmt, buff, 4), "b", 1);
+  is_str(my_fetch_str(hstmt, buff, 7), "t_bug19923c", 11);
+  is_str(my_fetch_str(hstmt, buff, 8), "b", 1);
+  is_str(my_fetch_str(hstmt, buff, 12), "first_constraint", 16);
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, buff, sizeof(buff), &len));
+  is_num(len, SQL_NULL_DATA);
+  ok_stmt(hstmt, SQLGetData(hstmt, 6, SQL_C_CHAR, buff, sizeof(buff), &len));
+  is_num(len, SQL_NULL_DATA);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug19923c, t_bug19923b, t_bug19923a");
+  return OK;
+}
+
+
 /*
   Bug #32864 MyODBC /Crystal Reports truncated table names,
              lost fields when names > 21chars
@@ -1142,15 +1208,17 @@ DECLARE_TEST(t_bug14407)
 DECLARE_TEST(t_bug32864)
 {
   SQLLEN dispsize= 0;
-  SQLLEN colsize= 0;
+  SQLULEN colsize= 0;
+  SQLCHAR dummy[20];
 
-  ok_stmt(hstmt, SQLTables(hstmt, "%", SQL_NTS, NULL, 0, NULL, 0, NULL, 0));
+  ok_stmt(hstmt, SQLTables(hstmt, (SQLCHAR *)"%", SQL_NTS, NULL, 0, NULL, 0,
+                           NULL, 0));
   ok_stmt(hstmt, SQLColAttribute(hstmt, 3, SQL_COLUMN_DISPLAY_SIZE, NULL, 0,
                                  NULL, &dispsize));
 
   is_num(dispsize, 64);
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 3, NULL, 0, NULL, NULL,
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 3, dummy, sizeof(dummy), NULL, NULL,
                                 &colsize, NULL, NULL));
 
   is_num(colsize, 64);
@@ -1170,8 +1238,8 @@ DECLARE_TEST(t_bug32989)
   ok_sql(hstmt, "drop table if exists t_bug32989");
   ok_sql(hstmt, "create table t_bug32989 (`doesn't work` int)");
 
-  ok_stmt(hstmt, SQLColumns(hstmt, "test", SQL_NTS, NULL, 0,
-                            "t_bug32989", SQL_NTS, NULL, 0));
+  ok_stmt(hstmt, SQLColumns(hstmt, (SQLCHAR *)"test", SQL_NTS, NULL, 0,
+                            (SQLCHAR *)"t_bug32989", SQL_NTS, NULL, 0));
   ok_stmt(hstmt, SQLFetch(hstmt));
 
   ok_stmt(hstmt, SQLGetData(hstmt, 4, SQL_C_CHAR, name, 20, &len));
@@ -1190,8 +1258,8 @@ DECLARE_TEST(t_bug32989)
 DECLARE_TEST(t_bug33298)
 {
   SQLRETURN rc= 0;
-  expect_stmt(hstmt, SQLProcedureColumns(hstmt, 0, NULL, 0, NULL,
-                                         0, NULL, 0, NULL),
+  expect_stmt(hstmt, SQLProcedureColumns(hstmt, NULL, 0, NULL, 0,
+                                         NULL, 0, NULL, 0),
                                          SQL_SUCCESS_WITH_INFO);
   rc= SQLFetch(hstmt);
   /**
@@ -1204,7 +1272,6 @@ DECLARE_TEST(t_bug33298)
   return OK;
 }
 
-
 /**
  Bug #12805: ADO failed to retrieve the length of LONGBLOB columns
 */
@@ -1213,6 +1280,7 @@ DECLARE_TEST(t_bug12805)
   SQLHENV    henv1;
   SQLHDBC    hdbc1;
   SQLHSTMT   hstmt1;
+  SQLCHAR    dummy[10];
   SQLULEN    length;  
   SQLUINTEGER len2;
 
@@ -1237,8 +1305,8 @@ DECLARE_TEST(t_bug12805)
 
   length= 0;
   ok_sql(hstmt1, "SELECT * FROM bug12805");
-  ok_stmt(hstmt1, SQLDescribeCol(hstmt1, 2, NULL, NULL, NULL, NULL,
-                                 &length, NULL, NULL));
+  ok_stmt(hstmt1, SQLDescribeCol(hstmt1, 2, dummy, sizeof(dummy) - 1, NULL,
+                                 NULL, &length, NULL, NULL));
   is_num(length, 2147483647);
 
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
@@ -1258,9 +1326,9 @@ DECLARE_TEST(t_bug12805)
 
   length= 0;
   ok_sql(hstmt, "SELECT * FROM bug12805");
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, NULL, NULL, NULL, NULL,
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, dummy, sizeof(dummy), NULL, NULL,
                                  &length, NULL, NULL));
-  is_num(length, 4294967295);
+  is_num(length, 4294967295UL);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_sql(hstmt, "DROP TABLE bug12805");
@@ -1305,8 +1373,8 @@ DECLARE_TEST(t_bug30770)
   ok_stmt(hstmt1, SQLExecDirect(hstmt1, buff, SQL_NTS));
 
   /* Get the info from just one table.  */
-  ok_stmt(hstmt1, SQLColumns(hstmt1, NULL, SQL_NTS, NULL, SQL_NTS, "bug30770",
-                            SQL_NTS, NULL, 0));
+  ok_stmt(hstmt1, SQLColumns(hstmt1, NULL, SQL_NTS, NULL, SQL_NTS,
+                             (SQLCHAR *)"bug30770", SQL_NTS, NULL, 0));
 
   ok_stmt(hstmt1, SQLFetch(hstmt1));
 
@@ -1321,6 +1389,54 @@ DECLARE_TEST(t_bug30770)
 }
 
 
+/**
+ Bug #36275: SQLTables buffer overrun
+*/
+DECLARE_TEST(t_bug36275)
+{
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
+                           (SQLCHAR *)
+/* Just a really long literal to blow out the buffer. */
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789"
+"0123456789012345678901234567890123456789012345678901234567890123456789",
+                           SQL_NTS));
+
+  return OK;
+}
+
+
+/*
+  Bug #39957 - NULL catalog did not return correct catalog in result
+*/
+DECLARE_TEST(t_bug39957)
+{
+  SQLCHAR buf[50];
+  ok_sql(hstmt, "drop table if exists t_bug39957");
+  ok_sql(hstmt, "create table t_bug39957 (x int)");
+  ok_stmt(hstmt, SQLTables(hstmt, NULL, 0, NULL, 0,
+			   (SQLCHAR *)"t_bug39957", SQL_NTS, NULL, 0));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  is_str(my_fetch_str(hstmt, buf, 1), "test", 5);
+  is_str(my_fetch_str(hstmt, buf, 3), "t_bug39957", 11);
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists t_bug39957");
+
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_columns_null)
   ADD_TEST(my_drop_table)
@@ -1343,11 +1459,14 @@ BEGIN_TESTS
   ADD_TEST(t_bug26934)
   ADD_TEST(t_bug29888)
   ADD_TEST(t_bug14407)
+  ADD_TEST(t_bug19923)
   ADD_TEST(t_bug32864)
   ADD_TEST(t_bug32989)
   ADD_TEST(t_bug33298)
   ADD_TEST(t_bug12805)
   ADD_TEST(t_bug30770)
+  ADD_TEST(t_bug36275)
+  ADD_TEST(t_bug39957)
 END_TESTS
 
 
