@@ -28,7 +28,7 @@ DECLARE_TEST(my_resultset)
     SQLRETURN   rc;
     SQLUINTEGER nRowCount=0;
     SQLULEN     pcColDef;
-    SQLCHAR     szColName[MAX_NAME_LEN];
+    SQLCHAR     szColName[MAX_NAME_LEN+1];
     SQLCHAR     szData[MAX_ROW_DATA_LEN+1];
     SQLSMALLINT nIndex,ncol,pfSqlType, pcbScale, pfNullable;
 
@@ -212,7 +212,8 @@ DECLARE_TEST(t_convert_type)
       mystmt(hstmt,rc);
 
       printMessage("CAST(MAX(col1) AS CHAR): %d", SqlType);
-      myassert(SqlType == SQL_VARCHAR || SqlType == SQL_LONGVARCHAR);
+      myassert(SqlType == SQL_VARCHAR || SqlType == SQL_LONGVARCHAR ||
+               SqlType == SQL_WVARCHAR || SqlType == SQL_WLONGVARCHAR);
 
       SQLFreeStmt(hstmt,SQL_CLOSE);
     }
@@ -318,11 +319,11 @@ DECLARE_TEST(t_desc_col)
   is(desc_col_check(hstmt, 15, "c15", SQL_LONGVARCHAR, 255, 255, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 16, "c16", SQL_LONGVARCHAR, 65535, 65535, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 17, "c17", SQL_LONGVARCHAR, 16777215, 16777215, 0,  SQL_NULLABLE) == OK);
-  is(desc_col_check(hstmt, 18, "c18", SQL_LONGVARCHAR, 4294967295 , 16777215 , 0,  SQL_NULLABLE) == OK);
+  is(desc_col_check(hstmt, 18, "c18", SQL_LONGVARCHAR, 4294967295UL, 16777215 , 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 19, "c19", SQL_LONGVARBINARY, 255, 255, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 20, "c20", SQL_LONGVARBINARY, 65535, 65535, 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 21, "c21", SQL_LONGVARBINARY, 16777215, 16777215, 0,  SQL_NULLABLE) == OK);
-  is(desc_col_check(hstmt, 22, "c22", SQL_LONGVARBINARY, 4294967295 , 16777215 , 0,  SQL_NULLABLE) == OK);
+  is(desc_col_check(hstmt, 22, "c22", SQL_LONGVARBINARY, 4294967295UL, 16777215 , 0,  SQL_NULLABLE) == OK);
   is(desc_col_check(hstmt, 23, "c23", SQL_LONGVARBINARY, 255, 5, 0,  SQL_NULLABLE) == OK);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -574,9 +575,7 @@ DECLARE_TEST(t_multistep)
     myassert(pcbValue == 99);
     myassert(szData[0] == 'A');
 
-
-    rc = SQLFetch(hstmt);
-    myassert(rc == SQL_NO_DATA_FOUND);
+    expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
 
     SQLFreeStmt(hstmt,SQL_UNBIND);
     SQLFreeStmt(hstmt,SQL_CLOSE);
@@ -619,22 +618,19 @@ DECLARE_TEST(t_zerolength)
     mystmt(hstmt,rc);
 
     pcbValue= pcbValue1= 99;
-    rc = SQLGetData(hstmt,1,SQL_C_CHAR,szData,0,&pcbValue);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szData, 0, &pcbValue));
     printMessage("length: %d", pcbValue);
     myassert(pcbValue == 0);
 
     bData[0]=bData[1]='z';
-    rc = SQLGetData(hstmt,2,SQL_C_BINARY,bData,0,&pcbValue1);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_BINARY, bData, 0, &pcbValue1));
     printMessage("length: %d", pcbValue1);
     myassert(pcbValue1 == 0);
     myassert(bData[0] == 'z');
     myassert(bData[1] == 'z');
 
     bData1[0]=bData1[1]='z';
-    rc = SQLGetData(hstmt,3,SQL_C_BINARY,bData1,0,&pcbValue2);
-    mystmt_err(hstmt,rc == SQL_SUCCESS_WITH_INFO,rc);
+    ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_BINARY, bData1, 0, &pcbValue2));
     printMessage("length: %d", pcbValue2);
     myassert(pcbValue2 == 0);
     myassert(bData1[0] == 'z');
@@ -652,7 +648,6 @@ DECLARE_TEST(t_zerolength)
     mystmt(hstmt,rc);
     printMessage("data: %s, length: %d", bData, pcbValue1);
     myassert(pcbValue1 == 0);
-    myassert(bData[0]== '\0');
 
     bData1[0]=bData1[1]='z';
     rc = SQLGetData(hstmt,3,SQL_C_CHAR,bData1,1,&pcbValue2);
@@ -786,7 +781,7 @@ DECLARE_TEST(t_cache_bug)
   ok_sql(hstmt, "CREATE TABLE t_cache (id INT)");
   ok_sql(hstmt, "INSERT INTO t_cache VALUES (1),(2),(3),(4),(5)");
 
-  sprintf((char *)conn, "DSN=%s;USER=%s;PASSWORD=%s;DATABASE=%s;OPTION=1048579",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;OPTION=1048579",
           mydsn, myuid, mypwd, mydb);
   if (mysock != NULL)
   {
@@ -853,7 +848,7 @@ DECLARE_TEST(t_non_cache_bug)
   ok_sql(hstmt, "CREATE TABLE t_cache (id INT)");
   ok_sql(hstmt, "INSERT INTO t_cache VALUES (1),(2),(3),(4),(5)");
 
-  sprintf((char *)conn, "DSN=%s;USER=%s;PASSWORD=%s;DATABASE=%s;OPTION=3",
+  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;OPTION=3",
           mydsn, myuid, mypwd, mydb);
   if (mysock != NULL)
   {
@@ -1478,7 +1473,7 @@ DECLARE_TEST(tmysql_rowstatus)
 {
     SQLRETURN rc;
     SQLHSTMT hstmt1;
-    SQLROWSETSIZE pcrow[4];
+    SQLULEN pcrow[4];
     SQLUSMALLINT rgfRowStatus[6];
     SQLINTEGER nData= 555;
     SQLCHAR szData[255] = "setpos-update";
@@ -1578,6 +1573,7 @@ DECLARE_TEST(t_true_length)
   SQLCHAR data1[25],data2[25];
   SQLLEN len1,len2;
   SQLULEN desc_len;
+  SQLSMALLINT name_len;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_true_length");
   ok_sql(hstmt, "CREATE TABLE t_true_length (a CHAR(20), b VARCHAR(15))");
@@ -1587,11 +1583,11 @@ DECLARE_TEST(t_true_length)
 
   ok_sql(hstmt, "SELECT * FROM t_true_length");
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, NULL, 0, NULL, NULL,
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, data1, sizeof(data1), &name_len, NULL,
                                 &desc_len, NULL, NULL));
   is_num(desc_len, 20);
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, NULL, 0, NULL, NULL,
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, data1, sizeof(data1), &name_len, NULL,
                                 &desc_len, NULL, NULL));
   is_num(desc_len, 15);
 
@@ -1738,13 +1734,13 @@ DECLARE_TEST(t_binary_collation)
                                 &decimal_digits, &nullable));
   if (mysql_min_version(hdbc, "5.2", 3) ||
       /* 5.0.46 or later in 5.0 series */
-      (!strncmp("5.0", server_version, 3) &&
+      (!strncmp("5.0", (char *)server_version, 3) &&
         mysql_min_version(hdbc, "5.0.46", 6)) ||
       /* 5.1.22 or later in 5.1 series */
-      (!strncmp("5.1", server_version, 3) &&
+      (!strncmp("5.1", (char *)server_version, 3) &&
         mysql_min_version(hdbc, "5.1.22", 6)))
   {
-    is_num(data_type, SQL_VARCHAR);
+    is_num(data_type, SQL_WVARCHAR);
   }
   else
   {
@@ -1761,7 +1757,7 @@ DECLARE_TEST(t_binary_collation)
 DECLARE_TEST(t_bug29239)
 {
   SQLHANDLE hstmt2;
-  SQLINTEGER xval;
+  SQLINTEGER xval = 88;
 
   ok_sql(hstmt, "drop table if exists bug29239");
   ok_sql(hstmt, "create table bug29239 ( x int )");
@@ -1782,6 +1778,8 @@ DECLARE_TEST(t_bug29239)
   is_num(xval, 44);
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
 
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
   ok_sql(hstmt, "drop table if exists bug29239");
   return OK;
 }
@@ -1846,6 +1844,120 @@ DECLARE_TEST(t_bug30958)
 }
 
 
+/**
+ A variant of the above test, with charset != ansi charset
+*/
+DECLARE_TEST(t_bug30958_ansi)
+{
+  SQLCHAR outbuf[20]= "bug";
+  SQLLEN outlen;
+  SQLINTEGER outmax= 0;
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+  ok_sql(hstmt, "CREATE TABLE bug30958 (tt_textfield TEXT CHARACTER SET latin2 NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO bug30958 (tt_textfield) VALUES ('')");
+
+  ok_sql(hstmt, "select tt_textfield from bug30958");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /*
+   check first that we get truncation, with zero bytes
+   available in out buffer, outbuffer should be untouched
+  */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* expect the same result, and not SQL_NO_DATA */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* now provide a space to read the data */
+  outmax= 1;
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax, &outlen));
+  is_num(outbuf[0], 0);
+  is_num(outlen, 0);
+
+  /* only now is it unavailable (test with empty and non-empty out buffer) */
+  outmax= 0;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+  outmax= 1;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+
+  return OK;
+}
+
+
+/**
+ A variant of the above tests using SQLWVARCHAR
+*/
+DECLARE_TEST(t_bug30958_wchar)
+{
+  SQLCHAR outbuf[20]= "bug";
+  SQLLEN outlen;
+  SQLINTEGER outmax= 0;
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+  ok_sql(hstmt, "CREATE TABLE bug30958 (tt_textfield TEXT NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO bug30958 (tt_textfield) VALUES ('')");
+
+  ok_sql(hstmt, "select tt_textfield from bug30958");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /*
+   check first that we get truncation, with zero bytes
+   available in out buffer, outbuffer should be untouched
+  */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* expect the same result, and not SQL_NO_DATA */
+  outlen= 99;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_SUCCESS_WITH_INFO);
+  is_str(outbuf, "bug", 3);
+  is_num(outlen, 0);
+  is(check_sqlstate(hstmt, "01004") == OK);
+
+  /* now provide a space to read the data */
+  outmax= sizeof(SQLWCHAR);
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax, &outlen));
+  is_num(outbuf[0], 0);
+  is_num(outlen, 0);
+
+  /* only now is it unavailable (test with empty and non-empty out buffer) */
+  outmax= 0;
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+  outmax= 1; /* outmax greater than 0, but less than sizeof(SQLWCHAR) */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, outmax,
+                                &outlen), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists bug30958");
+
+  return OK;
+}
+
+
 /**
  Bug #31246: Opening rowset with extra fields leads to incorrect SQL INSERT
 */
@@ -1893,6 +2005,10 @@ DECLARE_TEST(t_bug31246)
           sizeof(field3), NULL));
   ok_stmt(hstmt, SQLFetch(hstmt));
 
+  is_str(field1, buf, strlen((char *)buf) + 1);
+  is_num(field2, 10);
+  is_str(field3, "Default Text", 13);
+
   /* Clean-up */
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_sql(hstmt, "DROP TABLE t_bug31246");
@@ -1912,6 +2028,7 @@ DECLARE_TEST(t_bug13776)
   SQLULEN     pcColSz;
   SQLCHAR     szColName[MAX_NAME_LEN];
   SQLSMALLINT pfSqlType, pcbScale, pfNullable;
+  SQLLEN display_size, octet_length;
 
   SET_DSN_OPTION(1 << 27);
 
@@ -1928,6 +2045,14 @@ DECLARE_TEST(t_bug13776)
   /* Size of LONGTEXT should have been capped to 1 << 31. */
   is_num(pcColSz, 2147483647L);
 
+  /* also, check display size and octet length (see bug#30890) */
+  ok_stmt(hstmt1, SQLColAttribute(hstmt1, 1, SQL_DESC_DISPLAY_SIZE, NULL,
+                                  0, NULL, &display_size));
+  ok_stmt(hstmt1, SQLColAttribute(hstmt1, 1, SQL_DESC_OCTET_LENGTH, NULL,
+                                  0, NULL, &octet_length));
+  is_num(display_size, 2147483647L);
+  is_num(octet_length, 2147483647L);
+
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug13776");
@@ -1959,9 +2084,7 @@ DECLARE_TEST(t_bug13776_auto)
   SQLCHAR     szFileToLoad[255];
 
   /** @todo get the full path to the library using getenv */
-  env_path= getenv("CommonProgramFiles(x86)");
-  if (!env_path)
-    env_path= getenv("CommonProgramFiles");
+  env_path= getenv("CommonProgramFiles");
 
   if (!env_path)
   {
@@ -2009,6 +2132,144 @@ DECLARE_TEST(t_bug13776_auto)
 }
 
 
+/*
+  Bug#28617 Gibberish when reading utf8 TEXT column through ADO
+*/
+DECLARE_TEST(t_bug28617)
+{
+  SQLWCHAR outbuf[100];
+  SQLLEN outlen;
+
+  ok_sql(hstmt, "select 'qwertyuiop'");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, outbuf, 0, &outlen),
+              SQL_SUCCESS_WITH_INFO);
+  is_num(outlen, 10);
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, 0, &outlen),
+              SQL_SUCCESS_WITH_INFO);
+  is_num(outlen, 10 * sizeof(SQLWCHAR));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, outbuf, 100, &outlen));
+
+  is_num(outlen, 10 * sizeof(SQLWCHAR));
+  is(!memcmp(outbuf, W(L"qwertyuiop"), 11 * sizeof(SQLWCHAR)));
+
+  return OK;
+}
+
+
+/*
+  Bug#32684 - chunked retrieval of SQL_C_WCHAR fails
+*/
+DECLARE_TEST(t_bug32684)
+{
+  SQLWCHAR wbuf[20];
+  SQLCHAR abuf[20];
+  SQLLEN wlen, alen;
+  ok_sql(hstmt, "select repeat('x', 100), repeat('y', 100)");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  do
+  {
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, abuf,
+                              20, &alen));
+    printMessage("data= %s, len=%d\n", abuf, alen);
+  } while(alen > 20);
+
+  do
+  {
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, wbuf,
+                              20 * sizeof(SQLWCHAR), &wlen));
+    printMessage("data= %ls, len=%d\n", wbuf, wlen);
+  } while(wlen > 20 * sizeof(SQLWCHAR));
+
+  return OK;
+}
+
+
+/*
+  Bug #34271 - C/ODBC 5.1 does not list table fields in MSQRY32
+*/
+DECLARE_TEST(t_bug34271)
+{
+  SQLINTEGER x1= 0, x2= 0;
+
+  /* execute the query, but bind only the first column */
+  ok_sql(hstmt, "select 1,2");
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, &x1, 0, NULL));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(x1, 1);
+  is_num(x2, 0);
+  x1= 0;
+
+  /* unbind */
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+
+  /* execute the query, but bind only the second column */
+  ok_sql(hstmt, "select 1,2");
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_LONG, &x2, 0, NULL));
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_num(x1, 0);
+  is_num(x2, 2);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  return OK;
+}
+
+
+/*
+  Bug #34429 - SQLGetData gives incorrect data
+*/
+DECLARE_TEST(t_bug34429)
+{
+  SQLWCHAR buf[32];
+  SQLLEN reslen;
+
+  ok_sql(hstmt, "drop table if exists t_bug34429");
+  ok_sql(hstmt, "create table t_bug34429 (x varchar(200))");
+  ok_sql(hstmt, "insert into t_bug34429 values "
+                "(concat(repeat('x',32),repeat('y',32),repeat('z',16)))");
+  ok_sql(hstmt, "select x from t_bug34429");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* first chunk */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, buf, sizeof(buf),
+                                &reslen), SQL_SUCCESS_WITH_INFO);
+  printMessage("Chunk 1, len=%d, data=%ls", reslen, buf);
+  is_num(reslen, 80 * sizeof(SQLWCHAR));
+  is(!memcmp(buf, W(L"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"),
+             32 * sizeof(SQLWCHAR)));
+
+  /* second chunk */
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, buf, sizeof(buf),
+                                &reslen), SQL_SUCCESS_WITH_INFO);
+  printMessage("Chunk 2, len=%d, data=%ls", reslen, buf);
+  is_num(reslen, 49 * sizeof(SQLWCHAR));
+  is(!memcmp(buf, W(L"xyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy"),
+             32 * sizeof(SQLWCHAR)));
+
+  /* third chunk */
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_WCHAR, buf, sizeof(buf), &reslen));
+  printMessage("Chunk 3, len=%d, data=%ls", reslen, buf);
+  is_num(reslen, 18 * sizeof(SQLWCHAR));
+  is(!memcmp(buf, W(L"yyzzzzzzzzzzzzzzzz"), 18 * sizeof(SQLWCHAR)));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "drop table if exists t_bug34429");
+
+  return OK;
+}
+
+
 /*
   Bug #32420 - Don't cache results and SQLExtendedFetch work badly together
 */
@@ -2113,7 +2374,7 @@ DECLARE_TEST(t_bug24131)
   SQLLEN boundLen= 0;
   SQLULEN count;
   UWORD status;
-  SQLUINTEGER colSize;
+  SQLULEN colSize;
   SQLRETURN rc;
 
   ok_sql(hstmt, "drop table if exists bug24131");
@@ -2135,13 +2396,8 @@ DECLARE_TEST(t_bug24131)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"show create table bug24131", SQL_NTS));
 
-  rc= SQLDescribeCol(hstmt, 2, NULL, 0, NULL, NULL, &colSize, NULL, NULL);
-
-  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
-  {
-    printMessage("SQLDescribeCol returned %d", rc);
-    return FAIL;
-  }
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, buff, sizeof(buff), NULL, NULL,
+                                &colSize, NULL, NULL));
 
   ok_stmt(hstmt, SQLBindCol(hstmt,2,SQL_C_BINARY, buff, 1024, &boundLen));
 
@@ -2150,6 +2406,7 @@ DECLARE_TEST(t_bug24131)
 
   ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_NEXT, 1, &count, &status));
 
+  printMessage("colSize: %ld, boundLen: %ld", colSize, boundLen);
   is(colSize >= boundLen);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -2159,6 +2416,83 @@ DECLARE_TEST(t_bug24131)
 }
 
 
+/*
+  Bug #36069 - SQLProcedures followed by a SQLFreeStmt causes a crash
+ */
+DECLARE_TEST(t_bug36069)
+{
+  SQLSMALLINT size;
+
+  ok_stmt(hstmt, SQLProcedures(hstmt, NULL, 0, NULL, 0,
+                               (SQLCHAR *)"non-existing", SQL_NTS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &size));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"select ?", SQL_NTS));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+  ok_stmt(hstmt, SQLNumResultCols(hstmt, &size));
+
+  return OK;
+}
+
+
+/*
+  Bug #41942 - SQLDescribeCol() segfault with non-zero name length
+  and null buffer
+*/
+DECLARE_TEST(t_bug41942)
+{
+  SQLSMALLINT len;
+  ok_sql(hstmt, "select 1 as name");
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, NULL, 10, &len,
+                                NULL, NULL, NULL, NULL));
+  is_num(len, 4);
+  return OK;
+}
+
+
+/*
+  Bug 39644 - Binding SQL_C_BIT to an integer column is not working
+ */
+DECLARE_TEST(t_bug39644)
+{
+  char col1 = 0x3f;
+  char col2 = 0xff;
+  char col3 = 0x0;
+  char col4 = 0x1;
+
+  ok_sql(hstmt, "drop table if exists t_bug39644");
+  ok_sql(hstmt, "create table t_bug39644(col1 INT, col2 INT,"\
+	            "col3 BIT, col4 BIT)");
+
+  ok_sql(hstmt, "insert into t_bug39644 VALUES (5, 0, 1, 0)");
+
+  /* Do SELECT */
+  ok_sql(hstmt, "SELECT * from t_bug39644");
+
+  /* Now bind buffers */
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_BIT, &col1, sizeof(char), 0));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_BIT, &col2, sizeof(char), 0));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 3, SQL_C_BIT, &col3, sizeof(char), 0));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 4, SQL_C_BIT, &col4, sizeof(char), 0));
+
+  /* Fetch and check results */
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is( col1 == 1 );
+  is( col2 == 0 );
+  is( col3 == 1 );
+  is( col4 == 0 );
+
+  /* Clean-up */
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table t_bug39644");
+
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_resultset)
   ADD_TEST(t_convert_type)
@@ -2181,14 +2515,23 @@ BEGIN_TESTS
   ADD_TEST(t_bug16817)
   ADD_TEST(bug6157)
   ADD_TEST(t_binary_collation)
-  ADD_TODO(t_bug29239)
+  ADD_TEST(t_bug29239)
   ADD_TEST(t_bug30958)
+  ADD_TEST(t_bug30958_ansi)
+  ADD_TEST(t_bug30958_wchar)
   ADD_TEST(t_bug31246)
   ADD_TEST(t_bug13776)
   ADD_TEST(t_bug13776_auto)
+  ADD_TEST(t_bug28617)
+  ADD_TEST(t_bug32684)
+  ADD_TEST(t_bug34271)
+  ADD_TEST(t_bug34429)
   ADD_TEST(t_bug32420)
   ADD_TEST(t_bug34575)
   ADD_TEST(t_bug24131)
+  ADD_TEST(t_bug36069)
+  ADD_TEST(t_bug41942)
+  ADD_TEST(t_bug39644)
 END_TESTS
 
 
