@@ -36,6 +36,7 @@
 #ifdef WIN32
 #  include <windows.h>
 #  define sleep(x) Sleep(x*1000)
+#  include <crtdbg.h>
 #else
 #  include <unistd.h>
 #  include <signal.h>
@@ -52,6 +53,16 @@
 #include <time.h>
 
 
+/* Get routines for helping with Unicode conversion. */
+#define ODBCTAP
+
+typedef unsigned int UTF32;
+typedef unsigned short UTF16;
+typedef unsigned char UTF8;
+
+#include "../util/unicode_transcode.c"
+
+
 void printMessage(char *fmt, ...) {
   va_list ap;
   va_start(ap, fmt);
@@ -67,7 +78,7 @@ void printMessage(char *fmt, ...) {
 #define MAX_ROW_DATA_LEN 1000
 #define MYSQL_NAME_LEN 64
 
-SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 3.51 Driver}";
+SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 5.1 Driver}";
 SQLCHAR *mydsn= (SQLCHAR *)"test";
 SQLCHAR *myuid= (SQLCHAR *)"root";
 SQLCHAR *mypwd= (SQLCHAR *)"";
@@ -143,6 +154,18 @@ DWORD WINAPI win32_alarm(LPVOID arg)
 #define RUN_TESTS_ALARM  if (do_alarms) alarm(30)
 #endif
 
+void mem_debug_init()
+{
+#ifdef _WIN32
+  int dbg = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
+  dbg |= _CRTDBG_ALLOC_MEM_DF;
+  dbg |= _CRTDBG_CHECK_ALWAYS_DF;
+  dbg |= _CRTDBG_DELAY_FREE_MEM_DF;
+  dbg |= _CRTDBG_LEAK_CHECK_DF;
+  _CrtSetDbgFlag(dbg);
+#endif
+}
+
 #define BEGIN_TESTS my_test tests[]= {
 #define ADD_TEST(name) { #name, name, OK   },
 #define ADD_TODO(name) { #name, name, FAIL },
@@ -160,6 +183,8 @@ int main(int argc, char **argv) \
   SQLHSTMT hstmt; \
   int      i, num_tests, failcnt= 0; \
   ENABLE_ALARMS; \
+\
+  mem_debug_init(); \
 \
   /* Set from environment, possibly overrided by command line */ \
   if (getenv("TEST_DSN")) \
@@ -291,33 +316,36 @@ do { \
 
 
 /**
-  If calling a help function that doesn't return OK,
-  terminate as we would have using ok_stmt()
+  Verify that the results of an ODBC function call on a descriptor handle was
+  SQL_SUCCESS or SQL_SUCCESS_WITH_INFO.
 
-  @param call  The function call
+  @param desc Handle for descriptor object
+  @param call The function call
 */
-#define nok_pass_on(call) \
+#define ok_desc(desc, call) \
 do { \
-  int rc= (call); \
-  if (rc != OK) \
-    return rc; \
+  SQLRETURN rc= (call); \
+  print_diag(rc, SQL_HANDLE_DESC, (desc), #call, __FILE__, __LINE__); \
+  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
+    return FAIL; \
 } while (0)
 
 
 /**
   Verify that the result of an ODBC function call matches an expected
-  result, such as SQL_NO_DATA_FOUND.
+  result, such as SQL_ERROR or SQL_NO_DATA_FOUND.
 
-  @param statement Handle for statement object
+  @param hnd       Handle for object
+  @param type      Type of handle
   @param call      The function call
   @param expect    The expected result
 */
-#define expect_stmt(statement, call, expect) \
+#define expect_odbc(hnd, type, call, expect) \
 do { \
   SQLRETURN rc= (call); \
   if (rc != (expect)) \
   { \
-    print_diag(rc, SQL_HANDLE_STMT, (statement), #call, __FILE__, __LINE__); \
+    print_diag(rc, (type), (hnd), #call, __FILE__, __LINE__); \
     printf("# Expected %d, but got %d in %s on line %d\n", (expect), rc, \
            __FILE__, __LINE__); \
     return FAIL; \
@@ -325,6 +353,22 @@ do { \
 } while (0)
 
 
+#define expect_env(env, call, expect) \
+  expect_odbc((env), SQL_HANDLE_STMT, (call), (expect))
+
+
+#define expect_dbc(dbc, call, expect) \
+  expect_odbc((dbc), SQL_HANDLE_STMT, (call), (expect))
+
+
+#define expect_stmt(statement, call, expect) \
+  expect_odbc((statement), SQL_HANDLE_STMT, (call), (expect))
+
+
+#define expect_desc(desc, call, expect) \
+  expect_odbc((desc), SQL_HANDLE_DESC, (call), (expect))
+
+
 /**
   Verify that the results of an ODBC function call on an environment handle
   was SQL_SUCCESS or SQL_SUCCESS_WITH_INFO.
@@ -395,6 +439,25 @@ do { \
 } while (0);
 
 
+/**
+  Verify that a wide string (wchar_t *) matches an expected value.
+
+  @param a     The string to compare
+  @param b     The string to compare against
+  @param c     The number of characters to compare
+*/
+#define is_wstr(a, b, c) \
+do { \
+  wchar_t *val_a= (a), *val_b= (b); \
+  int val_len= (int)(c); \
+  if (memcmp(val_a, val_b, val_len * sizeof(wchar_t)) != 0) { \
+    printf("# %s ('%*ls') != '%*ls' in %s on line %d\n", \
+           #a, val_len, val_a, val_len, val_b, __FILE__, __LINE__); \
+    return FAIL; \
+  } \
+} while (0);
+
+
 /**
   Verify that a number (long integer) matches an expected value.
 
@@ -411,7 +474,12 @@ do { \
 } while (0);
 
 
-int check_sqlstate(SQLHSTMT hstmt, char *sqlstate)
+/* convenience macro for using statement */
+#define check_sqlstate(stmt, sqlstate) \
+  check_sqlstate_ex((stmt), SQL_HANDLE_STMT, (sqlstate))
+
+
+int check_sqlstate_ex(SQLHANDLE hnd, SQLSMALLINT hndtype, char *sqlstate)
 {
   SQLCHAR     sql_state[6];
   SQLINTEGER  err_code= 0;
@@ -419,7 +487,7 @@ int check_sqlstate(SQLHSTMT hstmt, char *sqlstate)
   SQLSMALLINT err_len= 0;
 
   memset(err_msg, 'C', SQL_MAX_MESSAGE_LENGTH);
-  SQLGetDiagRec(SQL_HANDLE_STMT, hstmt, 1, sql_state, &err_code, err_msg,
+  SQLGetDiagRec(hndtype, hnd, 1, sql_state, &err_code, err_msg,
                 SQL_MAX_MESSAGE_LENGTH - 1, &err_len);
 
   is_str(sql_state, (SQLCHAR *)sqlstate, 5);
@@ -433,7 +501,7 @@ int check_sqlstate(SQLHSTMT hstmt, char *sqlstate)
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 		       const char *text, const char *file, int line)
 {
-  if (!SQL_SUCCEEDED(rc))
+  if (rc != SQL_SUCCESS)
   {
     SQLCHAR     sqlstate[6], message[SQL_MAX_MESSAGE_LENGTH];
     SQLINTEGER  native_error;
@@ -559,6 +627,11 @@ static int my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
             disp_size = col_len;
         if (disp_size < 4 && nullable)
             disp_size = 4;
+        /*
+          We cap disp_size to avoid problems when we have problems with
+          it being very large, such as 64-bit issues in the driver.
+        */
+        disp_size= (disp_size > 512) ? 512 : disp_size;
         for (j=1; j < disp_size+3; j++)
           fprintf(stdout, "-");
         fprintf(stdout, "+");
@@ -590,6 +663,11 @@ static int my_print_data(SQLHSTMT hstmt, SQLUSMALLINT index,
         disp_size = col_len;
     if (disp_size < 4 && nullable)
         disp_size = 4;
+    /*
+      We cap disp_size to avoid problems when we have problems with
+      it being very large, such as 64-bit issues in the driver.
+    */
+    disp_size= (disp_size > 512) ? 512 : disp_size;
     if (length == SQL_NULL_DATA)
         fprintf(stdout, "%-*s  |", (int)disp_size, "NULL");
     else
@@ -655,7 +733,7 @@ SQLINTEGER myresult(SQLHSTMT hstmt)
     SQLUINTEGER nRowCount;
     SQLCHAR     ColName[MAX_NAME_LEN+1];
     SQLCHAR     Data[MAX_ROW_DATA_LEN+1];
-    SQLLEN      pcbLength, size;
+    SQLLEN      pcbLength;
     SQLUSMALLINT nIndex;
     SQLSMALLINT  ncol;
 
@@ -690,9 +768,6 @@ SQLINTEGER myresult(SQLHSTMT hstmt)
 
         for (nIndex=1; nIndex<= ncol; nIndex++)
         {
-            rc = SQLColAttribute(hstmt,nIndex,SQL_DESC_DISPLAY_SIZE,NULL,0,
-                                 NULL,&size);
-            mystmt(hstmt,rc);
             rc = SQLGetData(hstmt, nIndex, SQL_C_CHAR, Data,
                             MAX_ROW_DATA_LEN,&pcbLength);
             mystmt(hstmt,rc);
@@ -746,42 +821,68 @@ SQLRETURN tmysql_prepare(SQLHSTMT hstmt, char *sql_stmt)
 /**
   return integer data by fetching it
 */
-SQLINTEGER my_fetch_int(SQLHSTMT hstmt, SQLUSMALLINT irow)
+SQLINTEGER my_fetch_int(SQLHSTMT hstmt, SQLUSMALLINT icol)
 {
     SQLINTEGER nData;
     SQLLEN len;
 
-    SQLGetData(hstmt, irow, SQL_INTEGER, &nData, 0, &len);
+    SQLGetData(hstmt, icol, SQL_INTEGER, &nData, 0, &len);
     printMessage("my_fetch_int: %ld (%ld)", (long int)nData, len);
     return (len != SQL_NULL_DATA) ? nData : 0;
 }
+
+
 /**
   return string data, by fetching it
 */
-const char *my_fetch_str(SQLHSTMT hstmt, SQLCHAR *szData,SQLUSMALLINT irow)
+const char *my_fetch_str(SQLHSTMT hstmt, SQLCHAR *szData,SQLUSMALLINT icol)
 {
     SQLLEN nLen;
 
-    SQLGetData(hstmt,irow,SQL_CHAR,szData,MAX_ROW_DATA_LEN+1,&nLen);
+    SQLGetData(hstmt,icol,SQL_CHAR,szData,MAX_ROW_DATA_LEN+1,&nLen);
     printMessage(" my_fetch_str: %s(%ld)",szData,nLen);
     return((const char *)szData);
 }
 
-/*
-  Check if server running is MySQL
+
+/**
+  Convert a SQLWCHAR to wchar_t
 */
-int server_is_mysql(SQLHDBC hdbc)
+wchar_t *sqlwchar_to_wchar_t(SQLWCHAR *in)
 {
-    char driver_name[MYSQL_NAME_LEN];
+  static wchar_t buff[2048];
+  wchar_t *to= buff;
 
-    SQLGetInfo(hdbc,SQL_DRIVER_NAME,driver_name,MYSQL_NAME_LEN,NULL);
+  if (sizeof(wchar_t) == sizeof(SQLWCHAR))
+    return (wchar_t *)in;
 
-    if (strncmp(driver_name,"myodbc",6) >= 0 || strncmp(driver_name,"libmyodbc",9) >= 0)
-        return TRUE;
+  for ( ; *in && to < buff + sizeof(buff) - 2; in++)
+    to+= utf16toutf32((UTF16 *)in, (UTF32 *)to);
 
-    return FALSE;
+  *to= L'\0';
+
+  return buff;
 }
 
+
+/**
+  return wide string data, by fetching it and possibly converting it to wchar_t
+*/
+wchar_t *my_fetch_wstr(SQLHSTMT hstmt, SQLWCHAR *buffer, SQLUSMALLINT icol)
+{
+  SQLRETURN rc;
+  SQLLEN nLen;
+
+  rc= SQLGetData(hstmt, icol, SQL_WCHAR, buffer, MAX_ROW_DATA_LEN + 1, &nLen);
+  if (!SQL_SUCCEEDED(rc))
+    return L"";
+
+  buffer[nLen/sizeof(SQLWCHAR)]= 0;
+
+  return sqlwchar_to_wchar_t(buffer);
+}
+
+
 /*
   Check if driver supports SQLSetPos
 */
@@ -806,9 +907,6 @@ int mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
     SQLCHAR server_version[MYSQL_NAME_LEN+1];
     SQLRETURN rc;
 
-    if (!server_is_mysql(hdbc))
-        return TRUE;
-
     rc = SQLGetInfo(hdbc,SQL_DBMS_VER,server_version,MYSQL_NAME_LEN,NULL);
     mycon(hdbc, rc);
 
@@ -874,7 +972,7 @@ int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
 
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_DBC, *henv, hdbc));
 
-  sprintf((char *)connIn, "DSN=%s;USER=%s;PASSWORD=%s;DATABASE=test;OPTION=%d",
+  sprintf((char *)connIn, "DSN=%s;UID=%s;PWD=%s;DATABASE=test;OPTION=%d",
           (char *)mydsn, (char *)myuid, (char *)mypwd, myoption);
   if (mysock && mysock[0])
   {
@@ -917,6 +1015,62 @@ int free_basic_handles(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
 }
 
 
+/**
+ Helper for possibly converting a (wchar_t *) to a (SQLWCHAR *)
+*/
+#define W(string) dup_wchar_t_as_sqlwchar((string), \
+                                          sizeof(string) / sizeof(wchar_t))
+#define WL(string, len) dup_wchar_t_as_sqlwchar((string), (len))
+
+
+/**
+  Convert a wchar_t * to a SQLWCHAR * if a wchar_t is not the same as
+  SQLWCHAR. New space is allocated and never freed. Because this is used in
+  short-lived test programs, this is okay, if not ideal.
+*/
+SQLWCHAR *dup_wchar_t_as_sqlwchar(wchar_t *from, size_t len)
+{
+  if (sizeof(wchar_t) == sizeof(SQLWCHAR))
+  {
+    SQLWCHAR *to= malloc(len * sizeof(SQLWCHAR));
+    memcpy(to, from, len * sizeof(wchar_t));
+    return to;
+  }
+  else
+  {
+    size_t i;
+    SQLWCHAR *to= malloc(2 * len * sizeof(SQLWCHAR));
+    SQLWCHAR *out= to;
+    for (i= 0; i < len; i++)
+      to+= utf32toutf16((UTF32)from[i], (UTF16 *)to);
+    *to= 0;
+    return out;
+  }
+}
+
+
+/**
+ Helper for converting a (char *) to a (SQLWCHAR *)
+*/
+#define WC(string) dup_char_as_sqlwchar((string))
+
+
+/**
+  Convert a char * to a SQLWCHAR *. New space is allocated and never freed.
+  Because this is used in short-lived test programs, this is okay, if not
+  ideal.
+*/
+SQLWCHAR *dup_char_as_sqlwchar(SQLCHAR *from)
+{
+  SQLWCHAR *to= malloc((strlen((char *)from) + 1) * sizeof(SQLWCHAR));
+  SQLWCHAR *out= to;
+  while (from && *from)
+    *(to++)= (SQLWCHAR)*(from++);
+  *to= 0;
+  return out;
+}
+
+
 /**
   Check if we are using a driver manager for testing.
 
@@ -934,3 +1088,18 @@ int using_dm(HDBC hdbc)
 
   return 1;
 }
+
+/*
+  Check if we are using the unixODBC version specified
+*/
+int using_unixodbc_version(SQLHANDLE henv, SQLCHAR *ver)
+{
+  SQLCHAR buf[10];
+#ifdef SQL_ATTR_UNIXODBC_VERSION
+  if(SQLGetEnvAttr(henv, SQL_ATTR_UNIXODBC_VERSION, buf, 10, NULL) != SQL_SUCCESS)
+    return 0;
+  if(!strcmp(buf, ver))
+    return 1;
+#endif /* SQL_ATTR_UNIXODBC_VERSION */
+  return 0;
+}
